"use strict";
this.name        = "towbar";
this.author      = "Norby";
this.copyright   = "2013 Norbert Nagy";
this.licence     = "CC BY-NC-SA 3.0";
this.description = "Lock a derelict ship to the player ship and sell it in the dock.";
this.version     = "0.98";

//customizable properties
this.$TowbarDebug = false; //will spawn derelict ships and a weak offender right after undock
this.$TowbarFixed = true; //show towbar if no towed ship also, false works as retractable
this.$TowbarLimit127t = false; //can limit markets to the original capacity
this.$TowbarMaxReduct = 10; //how many hit will be reduced by Laser Reductor on a derelict ship
this.$TowbarMiner = 1; //0:offline, 1:cargo only, 2:up to full, 3:salvager
this.$TowbarNPCMissile = true; //NPCs with enhanced IQ: fire a missile before eject as last resort
this.$TowbarShow = true; //false change it to an invisible tractor beam
this.$TowbarSlowTurn = true; //set to false if cause jerky controls when tow a ship

//internal properties, should not touch
this.$TowbarBG = ""; //mission screen background
this.$TowbarCommonCargo = ["food", "textiles",  "minerals", "radioactives", "machinery",
	"furs", "computers", "luxuries"]; //no illegals, alloys and valuables, ordered by cost
	//"liquor_wines" left out to avoid _ in name
this.$TowbarCurrentRep = 0; //Store reputation to send message if step up one level
this.$TowbarInjectors = 0; //1 if there are offline injectors to give back when docked
this.$TowbarLaserReductorOn = true; //can turn on/off with activate
this.$TowbarLicenseFee = 20000; //must buy a license from the first incomes
this.$TowbarLostAlloys = []; //how many alloys lost during towing
this.$TowbarLostCargo = []; //how many cargo container lost during towing
this.$TowbarLostComps = []; //how many computers lost during towing
this.$TowbarMiningC = []; //mining counters, how many seconds after started
this.$TowbarMining1 = 20; //phase 1 end in seconds after mining started if player is not moving
this.$TowbarMining2 = 30; //phase 2 end in seconds after mining started if player is not moving
this.$TowbarMining3 = 50; //phase 3 end in seconds after mining started if player is not moving
this.$TowbarParcels = []; //store founded parcel names and descriptions, avoid reputation decrease if unfilled
this.$TowbarParcelExpired = 0; //store temporary how many found parcel expired for reputation restore, deprecated
this.$TowbarParcelFields = 6; //how many fields in Parcels array: pname, start, dest, atime, fee, galaxy
this.$TowbarParcelMin = 0; //parcels are expired at and below this index, search valid ones from here
this.$TowbarParcelMaxTime = 3600000; //maximum available to finish a parcel delivery (1000 hours)
this.$TowbarParcelReputation = -1000; //store temporary player's reputation to limit reputation restore, deprecated
this.$TowbarParcelTimer = null; //check expired parcels in every 5 minutes
this.$TowbarPitchC = 0; //pitch counter to break the towbar
this.$TowbarPrevMiner = this.$TowbarMiner; //save ship miner mode to detect change
this.$TowbarPrevOri = null; //used to slow pitch and yaw rates when towing
this.$TowbarPrevPos = null; //used to anchor to the too heavy ship
this.$TowbarPrevSpeed = 0; //ship miner check speed changes
this.$TowbarRepairC = 0; //repair counter, how many seconds left
this.$TowbarRunScreens = [];//need store more screens when a second Tug Drone hit but the first payout still shown
this.$TowbarShip = null; //the towed ship
this.$TowbarShipHealth = 0; //from 0 (damaged ship) to 1 (undamaged, no losses nor blow during towing)
this.$TowbarShips = []; //all towed ships to store cargo and alloy loss
this.$TowbarSound = null; //lock on soundsource
this.$TowbarSound2 = null; //break soundsource
this.$TowbarSound3 = null; //explosion soundsource
this.$TowbarSound4 = null; //sell soundsource
this.$TowbarSound5 = null; //applause soundsource
this.$TowbarTargetLost = false; //recheck player target after shipTargetLost event from FCB to show "Empty" faster
this.$TowbarFCB = null; //FrameCallBack updating towed ship
this.$TowbarFCB2 = null; //FrameCallBack updating ship in sell salvage screen
this.$TowbarTimer = null; //towbar timer to check near derelict ship
this.$TowbarV = null; //towbar visual effect

//world script events
this.startUp = function() {
	this.$TowbarInjectors = 0;
	this.$TowbarLockDelta = 0;
	this.$TowbarLostAlloys = [];
	this.$TowbarLostCargo = [];
	this.$TowbarLostComps = [];
	this.$TowbarMiningC = [];

	var mv = missionVariables.$TowbarParcels;
	if( !mv ) this.$TowbarParcels = []; 
	else if( (mv+"").indexOf(",") ) this.$TowbarParcels = (mv+"").split(","); //need +"" for bugfix
	else this.$TowbarParcels = [mv];

	this.$TowbarParcelMin = 0; //all parcels are expired below this index
	this.$Towbar_ParcelRemoveExpired( true ); //check parcels, set new ParcelMin
	this.$Towbar_ParcelTimer = new Timer(this, this.$Towbar_ParcelTimed, 60, 60);//check in every minutes
	this.$TowbarParcelExpired = 0; //store temporary how many found parcel expired for reputation restore
	this.$TowbarParcelReputation = -1000; //store temporary player's reputation to limit restore after expired removes
	
	this.$TowbarBG = ""; //mission screen background
	if(worldScripts["BGS-M"]) this.$TowbarBG = "bgs-i_equipship.png";
	else if(worldScripts["Better Screens"]) this.$TowbarBG = "bs-base-bg.png";
	
	this.$TowbarPitchC = 0;
	this.$TowbarPrevMiner = this.$TowbarMiner;
	this.$TowbarPrevOri = null;
	this.$TowbarPrevPos = null;
	this.$TowbarPrevSpeed = 0;
	this.$TowbarRepairC = 0;
	this.$TowbarRunScreens = [];
	this.$TowbarShip = null;
	this.$TowbarShips = [];
	this.$TowbarSound = new SoundSource;
	this.$TowbarSound.sound = "towbarlockon.ogg"; //sound of the lock on
	this.$TowbarSound.loop = false;
	this.$TowbarSound.repeatCount = 1;
	this.$TowbarSound2 = new SoundSource;
	this.$TowbarSound2.sound = "towbarbreak.ogg"; //sound of the towbar break
	this.$TowbarSound2.loop = false;
	this.$TowbarSound2.repeatCount = 1;
	this.$TowbarSound3 = new SoundSource;
	this.$TowbarSound3.sound = "bigbang.ogg"; //sound of the towed ship explosion
	this.$TowbarSound3.loop = false;
	this.$TowbarSound3.repeatCount = 1;
	this.$TowbarSound4 = new SoundSource;
	if( worldScripts["BGS-M"] ) this.$TowbarSound4.sound = "bgs-c_sell.ogg";  //bgs sell sound
	else this.$TowbarSound4.sound = "sell.ogg"; //sell sound
	this.$TowbarSound4.loop = false;
	this.$TowbarSound4.repeatCount = 1;
	this.$TowbarSound5 = new SoundSource;
	this.$TowbarSound5.sound = "towbarapplause.ogg"; //sound of the new level
	this.$TowbarSound5.loop = false;
	this.$TowbarSound5.repeatCount = 1;
	this.$TowbarSound6 = new SoundSource;
	this.$TowbarSound6.sound = "towbarhitderelict.ogg"; //sound if player hit a derelict ship
	this.$TowbarSound6.loop = false;
	this.$TowbarSound6.repeatCount = 1;
	this.$Towbar_UpdateReputation(true);
	
//	player.ship.awardEquipment("EQ_FUEL_INJECTION");//debug
//	player.ship.setEquipmentStatus("EQ_FUEL_INJECTION", "EQUIPMENT_DAMAGED" );//debug
//	player.credits += 1000;//debug

}

this.dayChanged = function(newday) {
//	this.$Towbar_ParcelLog(); //to check if chaged
//	this.$Towbar_ParcelReputationRestore();
	this.$Towbar_ParcelRemoveExpired( false, 0 );
}

this.equipmentDamaged = this.equipmentDestroyed = function(equipment) {
	if( equipment == "EQ_TOWBARINJ" && this.$TowbarShip && this.$TowbarShip.isValid
		&& player.ship.equipmentStatus("EQ_FUEL_INJECTION") == "EQUIPMENT_OK" ) {
		this.$TowbarInjectors = 1; //to give back when docked
		player.ship.setEquipmentStatus("EQ_FUEL_INJECTION", "EQUIPMENT_DAMAGED" );
		player.consoleMessage("Injectors offline", 5);
	}
	if( equipment == "EQ_TOWBAR" ) {
		if( player.ship.equipmentStatus("EQ_TOWBAR") == "EQUIPMENT_DAMAGED"
			&& player.ship.equipmentStatus("EQ_TOWBARREPAIRBOT") == "EQUIPMENT_OK" ) {
			player.consoleMessage("Towbar repair started", 5);
			player.ship.removeEquipment("EQ_TOWBARREPAIRBOT"); //one shot only
			this.$TowbarRepairC = 30+Math.floor(Math.random()*60); //30-90 sec
		}
		this.$TowbarShip = null;
		this.$Towbar_VClear();
		if( this.$TowbarInjectors == 1 &&
			player.ship.equipmentStatus("EQ_FUEL_INJECTION") == "EQUIPMENT_DAMAGED" ) {
			player.ship.setEquipmentStatus("EQ_FUEL_INJECTION", "EQUIPMENT_OK" );
			this.$TowbarInjectors = 0;
			player.consoleMessage("Injectors online", 5);
		}
	}
}

this.guiScreenWillChange = function(to, from) {
	if(!player.ship) return;
	var p = this.$TowbarParcels;
	if(to === "GUI_SCREEN_MANIFEST") {
		this.$TowbarParcelsShowTime = clock.seconds;
		for( var i = this.$TowbarParcelMin; i < p.length; i += this.$TowbarParcelFields ) {
			if( p[i+5] == galaxyNumber && p[i+4] > 0 && p[i+3] > this.$TowbarParcelsShowTime ) { //arrive in future
				player.ship.addParcel(p[i], p[i+1], p[i+2], p[i+3], p[i+4]);
			} //show find parcels until show manifest screen only to avoid reputation decrease when expire
		}
	} else if(from === "GUI_SCREEN_MANIFEST")
		for( var i = this.$TowbarParcelMin; i < p.length; i += this.$TowbarParcelFields )
			if( p[i+5] == galaxyNumber && p[i+4] > 0 && p[i+3] > this.$TowbarParcelsShowTime ) //arrive in future
				player.ship.removeParcel(p[i]); //remove all (from ParcelMin to avoid timelimit)
}

this.missionScreenOpportunity = function() {
	if( player.ship.docked && this.$TowbarShip && this.$TowbarShip.isValid 
		&& !this.$Towbar_IsHeavyShip( this.$TowbarShip ) ) //avoid heavy ships in aegis with autodock
		this.$Towbar_Payout(this.$TowbarShip);
}

this.playerBoughtEquipment = function(equipmentKey) {
	if( equipmentKey.substr(-7,7) == "_REFUND" ) { //any eq which end with _refund means full refund the eq named before
		player.ship.removeEquipment( equipmentKey ); //remove the "bought" refund eq
		var eq = equipmentKey.substr( 0, equipmentKey.length-7 );//the real eq
		if( player.ship.equipmentStatus( eq ) == "EQUIPMENT_OK" ) this.$Towbar_RefundEQ( eq );
		else if( player.ship.equipmentStatus( eq ) == "EQUIPMENT_DAMAGED" ) //need check to avoid message at refund
			player.consoleMessage("Need repair first"); //do not get full price back for a damaged eq
		return;
	}
}

this.playerStartedJumpCountdown = function(type, seconds) {
	this.$Towbar_Release(); //release towed ship

//follow lines moved into shipWillEnterWitchspace, here can force to remove all parcels by start and cancel galactic jump
//	if( type == "galactic" ) //need here due to playerEnteredNewGalaxy is too late and no playerWillEnterNewGalaxy
//		this.$Towbar_ParcelRemoveExpired( true, this.$TowbarParcelMaxTime+1 ); //remove all
	
}

this.playerWillSaveGame = function(message) {
	missionVariables.$TowbarParcels = this.$TowbarParcels;
}

this.reportScreenEnded = function() {
	this.$Towbar_UpdateReputation(false);
}

this.shipAttackedOther = function(other) { //player hits other
	if(!other || !other.isValid || other.isPlayer ) return; //exit if the target is died or player (for sure)
	if( !other.isDerelict && other.energy < 23 && other.energy > 0 && !other.isThargoid ) {
		//the next Military Laser hit can destroy the target
		if( this.$TowbarNPCMissile ) other.fireMissile();//last resort
		if( other.equipmentStatus("EQ_ESCAPE_POD") === "EQUIPMENT_OK"
			&& other.abandonShip()) {//eject to make more derelict ship but only if has Escape Pod
			if( worldScripts["detectors"] ) {
				var dn = other.script.$Detectors_Origname;//short name with version
			} else {
				var dn = other.displayName;
				other.displayName = "Derelict " + other.displayName;
			}
			player.consoleMessage(dn+" ejected!");
			if( other.script && !other.script.$AlreadyScored ) {
				other.script.$AlreadyScored = true;
				player.score++;
			}
		}
	}
	if( other.isDerelict && !other.isThargoid
		&& ( !other.script || !other.script.$TowbarDerelictAttack
			|| other.script.$TowbarDerelictAttack < this.$TowbarMaxReduct ) ) {
		player.consoleMessage("Derelict!", 10); //warn to stop fire if want to salvage it
		this.$TowbarSound6.play(); //audio feedback if derelict
	}
	if( other.isDerelict && !other.isThargoid
		&& player.ship.equipmentStatus("EQ_LASERREDUCTOR") == "EQUIPMENT_OK"
		&& this.$TowbarLaserReductorOn ) {
		if( other.script ) {
			if( !other.script.$TowbarDerelictAttack ) {
				other.script.$TowbarDerelictAttack = 1;
			} else {
				if( ++other.script.$TowbarDerelictAttack == this.$TowbarMaxReduct )
					player.consoleMessage("Laser Reductor disabled for "+other.displayName, 10);
			}
		}
		if( !other.script || other.script.$TowbarDerelictAttack <= this.$TowbarMaxReduct ) { //prevent invulnerability
			var h = 0;
//			if( other.scriptInfo ) {
//				h = parseInt(other.scriptInfo.hardarmour);
//				if( !( h > 0 ) ) h = 0; //bugfix against NaN
//			}
			if( player.ship.currentWeapon.equipmentKey == "EQ_WEAPON_MILITARY_LASER" )
				other.energy += 22 - h; //help avoid destroying the valuable ship when just derelicted
			else other.energy += 14 - h;//beam laser
			if( this.$TowbarDebug ) log("Towbar", "New target energy: "+other.energy+" h:"+h);
		}
	}
	this.$Towbar_Bounty( other );
	this.$Towbar_SetTimedBombToDerelict( other );
}

this.shipCollided = function(otherShip) {
	this.$Towbar_SteerTo( otherShip );
}

this.shipDockedWithStation = function(station) {
	this.$Towbar_ParcelLog(); //to check if chaged since shipWillDockWithStation
//	this.$Towbar_ParcelReputationRestore_old();//old deprecated method
	
/* see http://aegidian.org/bb/viewtopic.php?f=4&t=11857&start=30
	if( this.$player_ejected && player.ship.equipmentStatus("EQ_FULL_INSURANCE") == "EQUIPMENT_OK") {
		player.consoleMessage("Full Insurance give undamaged equipments into your new ship", 5);
		var eq = player.ship.equipment;
		for( var i=0; eq[i]; i++ )
			if( eq[i] && player.ship.equipmentStatus(eq[i].equipmentKey) == "EQUIPMENT_DAMAGED")
				player.ship.setEquipmentStatus(eq[i].equipmentKey, "EQUIPMENT_OK");
		player.ship.removeEquipment("EQ_FULL_INSURANCE");
	}
	this.$player_ejected = false;
*/
}

this.shipExitedWitchspace = function() {
	this.$Towbar_ParcelRemoveExpired( false );
//	this.$Towbar_ParcelLog();
//	if( this.$TowbarDebug ) player.ship.fuel = 7; //easy multiple jump for parcel test
//	this.$Towbar_ParcelReputationRestore();
}

this.shipLaunchedEscapePod = function(escapepod, passengers) {
//	this.$player_ejected = true;
	if( this.$TowbarDebug )
		log("Towbar", system.filteredEntities(this, function () { return true; }, player.ship, 25600));
 	this.$TowbarShip = null; //do not get reward when docked
}

this.shipLaunchedFromStation = function() {
	
	if( this.$TowbarDebug ) { //parcel test
		for(var i=0; i<3; i++) this.$Towbar_ParcelAddUrgent();
		for(var i=0; i<3; i++) this.$Towbar_ParcelAdd();
	}

//	player.ship.awardEquipment("EQ_FULL_INSURANCE"); //used for test only
// 	var eq = player.ship.equipment; //local variable, faster
//	for( var i=0; eq[i] && i < 5; i++ )
//		if( eq[i] && player.ship.equipmentStatus(eq[i].equipmentKey) == "EQUIPMENT_OK")
//			player.ship.setEquipmentStatus(eq[i].equipmentKey, "EQUIPMENT_DAMAGED");

	if( this.$TowbarDebug ) for(var i=0; this.$TowbarCommonCargo[i]; i++)
		system.mainStation.setMarketQuantity(this.$TowbarCommonCargo[i], 127); //debug

	this.$TowbarPitchC = 0; //always stabilized in dock
	this.$TowbarRepairC = 0; //finished repair in dock
	this.$Towbar_Start();
}

this.shipTargetAcquired = function(target) {
	if( target && target.isValid ) { //set lollipop color of the targeted derelict ship
		this.$Towbar_ScannerColor2( target, true ); //forced player target
		this.$Towbar_SetTimedBombToDerelict( target );
	}
}
	
this.shipTargetLost = function(target) { 
	if( target && target.isValid ) //set lollipop color of the targeted derelict ship
		this.$Towbar_ScannerColor(target);
	player.ship.removeEquipment("EQ_DTAEMP");
	player.ship.removeEquipment("EQ_DTAMIN");
	this.$TowbarTargetLost = true; //recheck player target from FCB to show "Empty" faster
}

this.shipWillDockWithStation = function(station) {
//	this.$Towbar_ParcelLog();
//	this.$Towbar_ParcelRemoveExpired( false, 0 );
	if( system && !system.isInterstellarSpace && station && station.isMainStation ) { //delivered parcels
		var p = this.$TowbarParcels;
		for( var i = this.$TowbarParcelMin; i < p.length; i += this.$TowbarParcelFields ) {
			if( p[i+5] == galaxyNumber && p[i+2] == system.ID && p[i+4] > 0 && p[i+3] > clock.seconds ) {
				//addParcel need future arrival time
				player.ship.addParcel(p[i], p[i+1], p[i+2], p[i+3], p[i+4]);
				p[i+3] = clock.seconds;
				if( this.$TowbarDebug ) log("Towbar", p[i]+" to "+System.systemNameForID(p[i+2])
					+" delivered at "+p[i+3]+" for "+formatCredits(Math.abs(p[i+4]), false, true));
				p[i+4] = -p[i+4]; //set to delivered (negative fee mean this)
				mission.unmarkSystem( { system: p[i+2], name: p[i], markerColor: "",
					markerScale: 2, markerShape: "" } );
				if( i == this.$TowbarParcelMin + this.$TowbarParcelFields )
					this.$TowbarParcelMin = i; //increase until continuosly expired
			}
		}
	}
	
	player.ship.removeEquipment("EQ_DTAEMP");
	player.ship.removeEquipment("EQ_DTAMIN");
	var ship = this.$TowbarShip;
	if( ship && ship.isValid && this.$Towbar_IsHeavyShip( ship ) )
		this.$Towbar_Release(); //need if pressed autodock when anchored to a heavy ship
	if( this.$TowbarRepairC > 0 ) { //finished repair in dock
		this.$TowbarRepairC = 0;
		player.ship.setEquipmentStatus( "EQ_TOWBAR", "EQUIPMENT_OK" );
	}
	this.$Towbar_VClear();
	if( isValidFrameCallback( this.$TowbarFCB ) )
		removeFrameCallback( this.$TowbarFCB );
	if( this.$TowbarTimer ) {
		this.$TowbarTimer.stop();
		delete this.$TowbarTimer;
	}
	if( this.$TowbarInjectors == 1 && //needed in case of mission ship deivery
		player.ship.equipmentStatus("EQ_FUEL_INJECTION") == "EQUIPMENT_DAMAGED" ) {
		player.ship.setEquipmentStatus("EQ_FUEL_INJECTION", "EQUIPMENT_OK" );
		this.$TowbarInjectors = 0;
	}
}

this.shipWillEnterWitchspace = function(cause) {
//	this.$Towbar_ParcelLog();
	var t = 0;
//	if( cause == "galactic jump" ) t = this.$TowbarParcelMaxTime + 1; //remove all - old method
	if( this.$TowbarDebug && system && !system.isInterstellarSpace && player.ship && player.ship.targetSystem ) {
		var myRoute = System.infoForSystem( galaxyNumber, system.ID )
			.routeToSystem( System.infoForSystem( galaxyNumber, player.ship.targetSystem ), "OPTIMIZED_BY_TIME");
		if( myRoute && myRoute.time > 0 ) t =  myRoute.time * 3600;//change from hours to seconds
		log("Towbar", "Hyperjump time:"+t+"s from "+System.systemNameForID(system.ID)
			+" to "+System.systemNameForID(player.ship.targetSystem)+" start at "+(clock.seconds+t)
			+" TowbarParcelReputation:"+this.$TowbarParcelReputation);
	}
//	this.$Towbar_ParcelRemoveExpired( false, t );
//	if( this.$TowbarDebug ) log("Towbar", "TowbarParcelReputation2:"+this.$TowbarParcelReputation);
	
	player.ship.removeEquipment("EQ_DTAEMP");
	player.ship.removeEquipment("EQ_DTAMIN");
	this.$TowbarShip = null; //release rammed ship if any
	this.$Towbar_VClear();
	if( isValidFrameCallback( this.$TowbarFCB ) )
		removeFrameCallback( this.$TowbarFCB );
	if( this.$TowbarTimer ) {
		this.$TowbarTimer.stop();
		delete this.$TowbarTimer;
	}
}

this.shipWillExitWitchspace = function() { //use this event due to shipExitedWitchspace is not working in v1.77
//	this.$Towbar_ParcelLog();
	this.$Towbar_ParcelRemoveExpired( false );
//	if( this.$TowbarDebug ) log("Towbar", "TowbarParcelReputation3:"+this.$TowbarParcelReputation);
//	this.$Towbar_ParcelReputationRestore();
//	if( this.$TowbarDebug ) log("Towbar", "TowbarParcelReputation4:"+this.$TowbarParcelReputation);
	
	this.$Towbar_Start(); //attach the visual effect to the new system
	this.$Towbar_UpdateReputation(false);
}

this.shipWillLaunchFromStation = function() {
	if( this.$TowbarDebug && system && !system.isInterstellarSpace ) { //demo targets
//		if( worldScripts["hardships"]) var datakey = "[hardpython]";
//		if( worldScripts["hardships"]) var datakey = "[hardcobra3]";
		var datakey = "pirate";
//		var datakey = "[python]";
//		var datakey = "[cobra3-trader]";
//		var datakey = "[morayMED]";
//		var datakey = "[adder]";
//		var datakey = "[anaconda]";
//		var datakey = "[tharglet]";
		var ships = system.addShips(datakey, 10,
			system.mainStation.position.add(system.mainStation.vectorForward.multiply( 4000 )), 500);
		for(var i=0; i<ships.length; i++) {
			if( worldScripts["shipversion"] ) {
				worldScripts["shipversion"].shipSpawned(ships[i]);
				if( !worldScripts["detectors"] ) {//detectors use different function to show the version number
					ships[i].displayName += worldScripts["shipversion"].$ShipVersion_Show(ships[i]); // 1 - 10
					//support to show (and log) names with version number without Detectors OXP also
					if( ships[i].script ) ships[i].script.$Detectors_Origname = ships[i].displayName;
				}
			}
			ships[i].awardEquipment("EQ_ESCAPE_POD");
			if(i == 0) { //make first ship to weak offender without injectors, need a few shot only to eject it
				ships[i].removeEquipment("EQ_FUEL_INJECTION");
				ships[i].bounty=33;
				ships[i].energy=33;
			} else ships[i].abandonShip(); //other ships derelict
		}
//		system.addShips("[superdredger]",1,ships[0].position,1000);
//		system.addShips("[dredger-ds]",1,ships[0].position,1000);
//		system.addShips("[dredgertransport]",1,ships[0].position,1000);
//		system.addShips("[more-machinery1]",1,ships[0].position,1000);
//		system.addShips("[more-machinery2]",1,ships[0].position,1000);
//		system.addShips("[more-machinery3]",1,ships[0].position,1000);
//		system.addShips("[more-machinery4]",1,ships[0].position,1000);
//		system.addShips("[more-alloys1]",1,ships[0].position,1000);
//		system.addShips("[more-alloys2]",1,ships[0].position,1000);
//		system.addShips("[more-alloys3]",1,ships[0].position,1000);
//		system.addShips("[more-alloys4]",1,ships[0].position,1000);
//		system.addShips("[more-alloys5]",1,ships[0].position,1000);
		//if( worldScripts["Anarchies"] )
//			system.addShips("anarchies-salvage-gang",1,
//			system.mainStation.position.add(system.mainStation.vectorForward.multiply( -20000 )), 500);
		var all = system.allShips; //log all equipments on all ships
		for( var a = 0; a < all.length; a++ ) {
			log("Towbar", all[a].displayName+" "+all[a].entityPersonality+" mass:"+Math.round(all[a].mass/1000)
				+"t maxCargo:"+all[a].cargoSpaceCapacity+"t ");
			var i=-1;//index in the eq array
			var eq = all[a].equipment; //local variable, faster
			if(all[a].isValid && !all[a].isPlayer) while(eq[++i]) {
				if( eq[i].name.length > 0 ) {
					log("Towbar", all[a].name+" "+(i+1)+". "+eq[i].name);
				} else {//skip noname eqs awarded by NumericHUD to avoid timeLimit error
					for( var j = i + 10; j < eq.length; j+=10 ) {
						if( eq[j].name.length != 0 ) {
							i = j - 10; //end of skip
							j = eq.length;
						}
					}
					while( i < eq.length && eq[i].name.length == 0 ) i++;
					i--;
				}
			}
		}
	}
}


//Towbar methods
this.$Towbar_AlloysFromShip = function( ship ) {
	return( Math.min(80, Math.round( 0.3 * ship.mass / 1000 ))); ///get max. 100t alloy
}

this.$Towbar_Bounty = function( ship ) {
	if( ship.isDerelict && ship.bounty > 0 ) {
		player.credits += ship.bounty;
		player.consoleMessage("Bounty: "+formatCredits(ship.bounty, true, true));
		player.consoleMessage("Total: "+formatCredits(player.credits, true, true));
		if( worldScripts["towbar"].$TowbarDebug ) log("Towbar", ship.displayName+" "+
			ship.entityPersonality+" bounty: "+formatCredits(ship.bounty, true, true));//debug
		if( ship.script ) ship.script.$TowbarBounty = ship.bounty;//save to award illegal cargo
		ship.bounty = 0;
	}
}

this.$Towbar_Cargo = function( ship ) {
	var cargo = { t:0, name:"food", v:0, vname:"gold", vunit:"kg" };
	var cnames = this.$TowbarCommonCargo; //no illegal goods nor valuables, ordered by cost
	var rnd = Math.random();
	var lostc = 0;
	var id = this.$TowbarShips.indexOf(this.$TowbarShip);
	if( id > -1 ) lostc = this.$TowbarLostCargo[id]; //id=-1 if catched by Tug Drone
	var cap = Math.min(120, ship.cargoSpaceCapacity); //max.120t
	if( this.$TowbarDebug ) log("Towbar", "cargoSpaceCapacity:"+cap+" "+ship.cargoSpaceCapacity);//debug
	if( cap > 1 && cap > lostc ) {
		if( ship.script && ship.script.$TowbarBounty > 100 * rnd ) {//high bounty get costly illegal cargo
			var icnames = ["slaves", "firearms", "narcotics"]; //ordered by price
			cargo.name = icnames[ Math.round( rnd * ( icnames.length - 1 ) ) ];
			cargo.t = Math.min(cap, Math.round( (1 - rnd) * cap ) );
		} else { //common cargo
			cargo.name = cnames[ Math.round( rnd * ( cnames.length - 1 ) ) ];
			if( cargo.name == "food" ) cargo.t = cap; //too cheap, compensate with always max
			else {
				cargo.t = Math.min(cap, Math.round(
				1 + Math.random()*400 / (cnames.length*rnd+1))); //more t if cargo is cheap
			}
		}
	}
	var minpers = 26215;  // 20% of the ships, top 20% with ShipVersion OXP
	if( ship.entityPersonality > minpers ) { //add valuables
		var v = 0;
		if( rnd < 0.3 ) { cargo.vname = "gold"; cargo.vunit = "kg"; v = 60; } //6% carry Gold
		else if( rnd < 0.6 ) { cargo.vname = "platinum"; cargo.vunit = "kg"; v = 30; } //6% carry Platinum
//		else if( rnd < 0.6 ) { cargo.vname = "gem_stones"; cargo.vunit = "g"; v = 90; } //Gem-stones removed to avoid _ in name
		else if( cap > 1 && cap > lostc ) { //8% carry many normal cargo from machinery
			cargo.name = cnames[ Math.round( rnd * ( cnames.length - 6 ) )  + 5 ];
			cargo.t = Math.min(120, Math.min(cap, 1+Math.round( 10+cap*rnd )));
		}
		if( v > 0 ) {
			if( ship.mass < 130000 ) v = v / 3; //less in small ships
			cargo.v = Math.min(100, 10+Math.round( v*Math.random() ));
		}
	}
	if( id > -1 ) cargo.t = Math.max( 0, cargo.t - lostc );
	return( cargo );
}

this.$Towbar_Computers = function( ship ) {
	var t = 1; //ships usually has one main computer
	if( ship.mass > 100000000 ) t = 10; //baseship has 10 built-in computers
	else if( ship.mass > 1000000 ) t = 5; //big ships has 5 built-in computers (like Rocket Freighter)
	else if( ship.mass >= 130000 ) t = 2; //large ships has 2 computers
	else if( ship.mass < 30000 ) t = 0; //small ships has microcomputers only
	var id = this.$TowbarShips.indexOf(this.$TowbarShip);
	if( id > -1 ) t = Math.max( 0, t - this.$TowbarLostComps[id] );
	return(t);
}

this.$Towbar_EqPrice = function( ship ) {
	var eq = ship.equipment; //local variable, faster
	var eprice = 0;
	var i = -1;
	while( eq[++i] ) eprice += Math.min(175, eq[i].price / 200); //average payment for an equipment in the dock
	var w = ship.forwardWeapon;
	if( w ) eprice += Math.round( w.price/10 * ( 0.1 + Math.random() * 0.1 ) ); //min. 10%, max. 20%
	w = ship.aftWeapon;
	if( w ) eprice += Math.round( w.price/10 * ( 0.1 + Math.random() * 0.1 ) ); //min. 10%, max 20%
	return( eprice );
}

this.$Towbar_FCB = function( delta ) { //FrameCallBack updating towed ship and Derelict display
	if( !player.ship || !player.ship.isValid ) return; //exit if player died
	var r = worldScripts["towbar"];
	
	if( r.$TowbarTargetLost ) { //recheck player target after shipTargetLost event from FCB to show "Empty" faster
		r.$TowbarTargetLost = false;
		if( player.ship.target && player.ship.target.isValid ) //set lollipop color of the targeted derelict ship
			r.$Towbar_ScannerColor2(player.ship.target, true); //forced player target
	}

	if( r.$TowbarRunScreens.length > 0 && guiScreen != "GUI_SCREEN_MISSION" ) {
		var rs = r.$TowbarRunScreens.shift(); //get and remove the oldest element
		r.$Towbar_RunScreen(rs.place, rs.msg, rs.model); //show sell salvage screen
	}
	if( !worldScripts["detectors"] ) {
		var eq = "EQ_DTADER";
		if( EquipmentInfo.infoForKey( eq ) ) {
			var who = player.ship.target;
			if( who && who.isValid && who.isDerelict && who != r.$TowbarShip )
				player.ship.awardEquipment( eq );
			else player.ship.removeEquipment( eq ); //remove always to clear when lost target
		}
	}
	if( r.$TowbarShip ) {
		if( r.$TowbarShip.isValid ) {
			var ma = Math.min( 2, player.ship.mass / r.$TowbarShip.mass ); ///small ship max. 2x
			if( r.$Towbar_IsHeavyShip( r.$TowbarShip ) ) {//anchor to the too heavy ship
				if( r.$TowbarPrevPos ) player.ship.position = r.$TowbarPrevPos; //can not move
				else {
					r.$TowbarPrevPos = player.ship.position; //first time save the position
					player.consoleMessage("You are anchored to a heavy ship", 5);
				}
			}
			var stable = true;
			var prevori = r.$TowbarPrevOri;
			if( prevori ) { //slow pitch and yaw rates when towing
				var ato = player.ship.heading.angleTo( prevori.vectorForward() );
//				player.consoleMessage(ato);//debug
//				player.consoleMessage(t.$TowbarPrevOri +" vs "+ player.ship.heading);//debug
				if( ato > 0.0005 ) {
					if( r.$TowbarPrevPos ) {//release the too heavy ship when steer
						r.$Towbar_Release();
						r.$TowbarPrevPos = null; //release anchor
						return; //to avoid orienting the released ship, etc.
					}
					var wst = worldScripts["telescope"];//must prevent collision with auto steering
					if( r.$TowbarSlowTurn //set to false if cause jerky controls when tow a ship
						&& (!wst || !isValidFrameCallback( wst.$TelescopeSteerFCB ))) {
						var a = ato * ma / 3; // 1/3 of the original rate with same mass, min. 1/5 max. 2/3
//						player.consoleMessage("Towbar slow steering "+ato);//debug
						var c = prevori.vectorForward().cross( player.ship.heading ).direction();
						player.ship.orientation = prevori.rotate( c, -a );
					}
					
					if( player.ship.speed == 0 ) r.$TowbarPitchC += ato/2; //stopped ship count half
					else if( player.ship.speed > player.ship.maxSpeed //injectors count 1-7 times more
						&& player.ship.speed <= player.ship.maxSpeed * 7 )
						r.$TowbarPitchC += ato * player.ship.speed / player.ship.maxSpeed;
					else r.$TowbarPitchC += ato;

					var rnd = Math.random() - r.$TowbarPitchC / (20000 * ma);//small ship max. 2x
						//enough for +1-2 full 360 degree turn if towing the same mass
//					player.consoleMessage(Math.round(20*ma)+" "+Math.round(r.$TowbarPitchC*100)/100);
					if( rnd < delta*2 ) r.$Towbar_Lose( true );//lose some from the towed ship when steer
					
					if( r.$TowbarPitchC > 40*ma ) { //at least the first 720 degree turn is safe
						player.consoleMessage("Towbar unstable!");
						stable = false;
						if( rnd < 0 || r.$TowbarPitchC > 60*ma ) { //max. 3x
							rnd = Math.random();
							if( rnd < 0.3 * (1-r.$TowbarShipHealth)) {//max.30% to blow the ship
								r.$Towbar_ShipExplode();
							} else if( rnd > 0.7 ) { //30% chance to destroy the towbar
								player.consoleMessage("Towbar destroyed!");
								player.ship.removeEquipment("EQ_TOWBAR");
								r.equipmentDestroyed("EQ_TOWBAR");
								r.$TowbarSound2.play();//towbar break sound
							} else { //min.40% chance to break the towbar
								player.consoleMessage("Towbar broken!");
									player.ship.setEquipmentStatus("EQ_TOWBAR", "EQUIPMENT_DAMAGED");
								r.$TowbarSound2.play();//towbar break sound
							}
						}
					} else if( r.$TowbarPitchC > 38*ma )
						player.consoleMessage("Towbar will be unstable soon");
				}
			}
			if( stable ) {
				var m = -200;
				r.$TowbarShip.orientation = player.ship.orientation;
			} else {
				var m = -200 + rnd * r.$TowbarPitchC / 30;  ///show some unstability when changing direction
				r.$TowbarShip.orientation = prevori;
			}
			r.$TowbarShip.position = player.ship.position.add( player.ship.vectorForward.multiply( m ) );
		} else {
			r.$TowbarShip = null; //towed ship destoyed
			if( !r.$TowbarFixed ) r.$Towbar_VClear();
			r.$TowbarPrevPos = null; //release anchor
		}
		r.$TowbarPrevOri = player.ship.orientation;
	} else	r.$TowbarPrevPos = null; //release anchor
	if( r.$TowbarV ) {
		r.$TowbarV.position = player.ship.position;
		r.$TowbarV.orientation = player.ship.orientation;
	}
}

this.$Towbar_FCB2 = function( delta ) { //FrameCallBack updating ship in sell salvage screen
	if( mission.displayModel && mission.displayModel.isValid )
		mission.displayModel.orientation = mission.displayModel.orientation.rotateY( delta );
}

this.$Towbar_IsHeavyShip = function( ship ) { //too heavy to the curent player ship?
	if( player.ship.mass * 1.6 > ship.mass ) return false;
	return true;
}

this.$Towbar_IsDerelict = function(entity) {
	if( entity && entity.isValid && entity.isDerelict ) return true;
	return false;
}

this.$Towbar_Lock = function( otherShip ) { //chance to lose something from the towed ship
	var ws = worldScripts["towbar"];
	if( player.ship.equipmentStatus("EQ_TOWBAR") == "EQUIPMENT_OK" &&
		otherShip && otherShip.isValid && otherShip.isDerelict &&
		otherShip != ws.$TowbarShip ) {
		if( !ws.$Towbar_IsHeavyShip( otherShip ) //not too heavy or mining
			|| player.ship.equipmentStatus("EQ_SHIPMINER") == "EQUIPMENT_OK" ) {
			var e = worldScripts["exhibitions"];
			if( e && e.$ExhibitionsShips && e.$ExhibitionsShips.indexOf(otherShip) > -1 ) {
				player.consoleMessage("Towbar can not lock on ships in Exhibition",5);
				return;
			}
			if( e && e.$ExhibitionsSalonShips && e.$ExhibitionsSalonShips.indexOf(otherShip) > -1 ) {
				player.consoleMessage("Towbar can not lock on ships in Salon",5);
				return;
			}
				
			if( ws.$Towbar_ShipExplodeDamageSize( otherShip ) > player.ship.energy
				+ Math.min( player.ship.forwardShield, player.ship.aftShield ) ) {
				player.consoleMessage("Towbar can not autolock on "
					+ws.$Towbar_Name( otherShip )+", need more energy to survive a large trap", 5);
				return;
			}
			if( ws.$TowbarShip && ws.$TowbarShip.isValid ) //relock to a new ship: first release the old one
				player.consoleMessage("Towbar released "+ws.$Towbar_Name( ws.$TowbarShip ), 5);
			if( !otherShip.script ) otherShip.script = "oolite-default-ship-script.js";
			ws.$TowbarShip = otherShip;
			ws.$TowbarPrevPos = null; //release anchor for sure
			ws.$TowbarShip.velocity = Vector3D(0, 0, 0); //can not fix engine flare to the witchpoint
			ws.$TowbarShip.beaconCode = "D"; //Derelict
			if( !ws.$TowbarFixed ) ws.$Towbar_Show();
			if( player.ship.target == otherShip && worldScripts["telescope"] ) {
				worldScripts["telescope"].$TelescopeListi = 0;  //avoid spinning telescope model
				player.ship.target = null;
			}
			player.consoleMessage("Towbar locked on "+ws.$Towbar_Name( otherShip ), 5);
			var id = ws.$TowbarShips.indexOf(ws.$TowbarShip);
			if( id == -1 ) {
				ws.$TowbarShips.push(ws.$TowbarShip);//save to store alloy and cargo loss
				id = ws.$TowbarShips.indexOf(ws.$TowbarShip);
				ws.$TowbarLostAlloys[id] = 0;
				ws.$TowbarLostCargo[id] = 0;
				ws.$TowbarLostComps[id] = 0;
				ws.$TowbarMiningC[id] = 0;
			}
			if(ws.$TowbarShip.script && ws.$TowbarShip.script.$TowbarShipHealth > 0 )
				ws.$TowbarShipHealth = Math.min(1, ws.$TowbarShip.script.$TowbarShipHealth); //max.1
			else ws.$TowbarShipHealth = 0;//damaged ship
			if( player.ship.equipmentStatus("EQ_FUEL_INJECTION") == "EQUIPMENT_OK"
				&& player.ship.equipmentStatus("EQ_TOWBARINJ") != "EQUIPMENT_OK" ) { //inj.compatible eq.
				ws.$TowbarInjectors = 1; //to give back when docked
				player.ship.setEquipmentStatus("EQ_FUEL_INJECTION", "EQUIPMENT_DAMAGED" );
				player.consoleMessage("Injectors offline", 5);
			}
			ws.$TowbarSound.play();//lock on sound
			ws.$Towbar_Bounty( otherShip ); //get some if the missile launched at eject hit the player
			
			if( ws.$TowbarShip.script && ws.$TowbarShip.script.$TowbarMission )
				worldScripts["towbarmissions"].$TowbarMissions_Locked();//send message if mission ship
			else 	{
				if( ws.$TowbarMiningC[ id ] == 0 ) { //trapship check only at first lock, from ShipVersion oxp
					var sv = ( Math.floor( ws.$TowbarShip.entityPersonality / 328 ) + 1 ) / 10;
					var rnd = Math.random();
					if( rnd < sv / 100 ||  //0-10% or timed bomb already set
						ws.$TowbarShip.script && ws.$TowbarShip.script.$Towbar_TimedBombCounter > -1 ) {
						if( rnd < 0.001 * sv || sv > this.$TowbarCurrentRep ) //at least 10% to fail untrap
							this.$Towbar_ShipExplode();
						else {
							var t = "trap";
							if( ws.$TowbarShip.script &&
								ws.$TowbarShip.script.$Towbar_TimedBombCounter > -1 ) {
								ws.$TowbarShip.script.$Towbar_TimedBombCounter = -1;
								t = "timed bomb";
							}
							player.consoleMessage("Your salvager experience helped"
								+" to neutralize a "+t+" in "+ws.$Towbar_Name( ship ), 10);
						}
					}
				}
			}
			ws.$Towbar_Mining(); //start or continue
		} else { //too heavy and no ship miner
			player.consoleMessage("Towbar can not lock on "
				+ws.$Towbar_Name( otherShip )+": too heavy for your ship", 5);
		}
	}
}

this.$Towbar_Lose = function( steer ) { //chance to lose something from the towed ship
	var r = worldScripts["towbar"];
	if( r.$TowbarShip && r.$TowbarShip.isValid //chance to lost eqs, cargo or alloys over time from the towed ship
		&& ( steer || player.ship.speed > 0 ) ) { //no loss if stopped
		var rnd = Math.random();
		var sp = 1;
		//smaller chance at slow speeds, larger at injector speed, torus and steer like normal maxSpeed
		if( !steer && player.ship.speed <= player.ship.maxSpeed * 7 )
			sp = player.ship.speed / player.ship.maxSpeed; ///
		sp = sp * (1-r.$TowbarShipHealth); //healthy ship lose fewer or zero
		if( r.$TowbarPrevPos || sp < 0.1 ) return; //no loss if anchored or at tiny speed
		if( player.ship.speed <= player.ship.maxSpeed ) rnd += 0.004; //no multiple eq loss at normal speed
		if( rnd < 0.004 * sp ) { //lost more eq in every 250 sec
			var eq = r.$TowbarShip.equipment;
			if( eq.length >= 3 ) { //levae in the last equipment
				var lost = Math.max(2, Math.ceil(Math.random() * (eq.length-1) / 2)); //at least 2
				if( player.ship.equipmentStatus("EQ_SHIPMINERASSISTANT") == "EQUIPMENT_OK" )
					lost = Math.max(2, Math.floor( lost / 2 ) ); //reduce the lost
				if( Math.random() > 0.1 ) {
					player.consoleMessage(lost+"t Machinery flying from the towed ship", 5);
					var a = r.$TowbarShip.spawnOne("more-machinery");
					if( a && a.isValid ) {
						a.displayName = lost+"t Machinery";
						if( !a.script ) a.script = "oolite-default-ship-script.js";
						if( a.script ) {
							a.script.$cargo = lost;
							a.script.$cargoname = "machinery";
						}
					} else r.$TowbarShip.spawn("more-wreckage", Math.min(64, lost));
				} else player.consoleMessage(lost+" equipments destroyed in the towed ship", 5);
				for(var i = 0; i < lost; i++ ) {
					r.$TowbarShip.removeEquipment(eq[i].equipmentKey);
				}
			}
		} else if( rnd < 0.0045 * sp ) { //break towbar in every 2000s (no destroy nor ship explosion from movement)
			var ma = player.ship.mass / r.$TowbarShip.mass; //smaller ship has smaller chance to break the towbar
			if( rnd - 0.004 * sp < 0.0005 * sp / ma ) {
				player.consoleMessage("Towbar broken!");
				player.ship.setEquipmentStatus("EQ_TOWBAR", "EQUIPMENT_DAMAGED");
				r.$TowbarSound2.play();//towbar break sound
			}
		} else if( rnd < 0.01 * sp ) { //lost more than one unit cargo in every 200 sec
			var id = r.$TowbarShips.indexOf(r.$TowbarShip);
			if( id > -1 ) {
				var prevlost = 0;
				if(r.$TowbarLostCargo[id] > 0) prevlost = r.$TowbarLostCargo[id];
				var left = Math.min(20, r.$TowbarShip.cargoSpaceCapacity / 6) - prevlost; //lost max.20t
				if( left >= 2 ) {
					var lost = 2 + Math.round(Math.random() * (left-2) / 4);
					if( player.ship.equipmentStatus("EQ_SHIPMINERASSISTANT") == "EQUIPMENT_OK" )
						lost = Math.max(2, Math.floor( lost / 2 ) ); //reduce the lost
					if( Math.random() > 0.1 ) {
						var cname = "cargo";
						var a = r.$TowbarShip.spawnOne("more-cargo");
						if( a && a.isValid ) {
							var cnames = r.$TowbarCommonCargo;
							cname = cnames[Math.floor(Math.random() * cnames.length)];
							if( !a.script ) a.script = "oolite-default-ship-script.js";
							if( a.script ) {
								a.script.$cargo = lost;
								a.script.$cargoname = cname;
							}
							//if(system && system.mainStation && system.mainStation.market)
							//	cname = system.mainStation.market[cname].displayName;
							a.displayName = lost+"t "+cname;
						} else r.$TowbarShip.spawn("cargopod", Math.min(64, lost));
						player.consoleMessage(lost+"t "+cname+" flying from the towed ship", 5);
					} else player.consoleMessage(lost+"t Cargo destroyed in the towed ship", 5);
					r.$TowbarLostCargo[id] += lost;
				}
			}
		} else if( rnd < 0.03 * sp ) { //lost more alloys in every 50 sec
			var id = r.$TowbarShips.indexOf(r.$TowbarShip);
			if( id > -1 ) {
				var prevlost = 0;
				if(r.$TowbarLostAlloys[id] > 0) prevlost = r.$TowbarLostAlloys[id];
				var left = r.$Towbar_AlloysFromShip( r.$TowbarShip ) / 1.5 - prevlost;
				if( left >= 2 ) {
					var lost = 2 + Math.round(Math.random() * (left-2) / 2);
					if( player.ship.equipmentStatus("EQ_SHIPMINERASSISTANT") == "EQUIPMENT_OK" )
						lost = Math.max(2, Math.floor( lost / 2 ) ); //reduce the lost
					if( Math.random() > 0.1 ) {
						if( lost > 10 ) { //can not scoop too large pieces
							r.$TowbarShip.spawnOne("more-wreckage");
							player.consoleMessage(lost+"t Wreckage flying from the towed ship", 5);
						} else {
							player.consoleMessage(lost+"t Alloys flying from the towed ship", 5);
							var a = r.$TowbarShip.spawnOne("more-alloys");
							if( a && a.isValid ) {
								a.displayName = lost+"t Alloys";
								if( !a.script ) a.script = "oolite-default-ship-script.js";
								if( a.script ) {
									a.script.$cargo = lost;
									a.script.$cargoname = "alloys";
								}
							} else r.$TowbarShip.spawn("alloy",  Math.min(64, lost));
						}
					} else player.consoleMessage(lost+"t Alloys destroyed in the towed ship", 5);
					r.$TowbarLostAlloys[id] += lost;
				}
			}
		} else if( rnd < 0.04 * sp ) { //lost an eq in every 100 sec
			var eq = r.$TowbarShip.equipment;
			if( eq.length > 1 ) { //leave in the last equipment
				var num = Math.round(Math.random() * (eq.length-1));
				if( Math.random() > 0.1 ) {
					player.consoleMessage(eq[num].name+" flying from the towed ship", 5);
					var a = r.$TowbarShip.spawnOne("one-machinery");
					if( a && a.isValid ) {
						if( !a.script ) a.script = "oolite-default-ship-script.js";
						if( a.script ) {
							a.script.$cargo = 1;
							a.script.$cargoname = "machinery";
						}
						a.displayName = "Damaged "+eq[num].name;
					} else r.$TowbarShip.spawnOne("more-wreckage");
				} else player.consoleMessage(eq[num].name+" destroyed in the towed ship", 5);
				r.$TowbarShip.removeEquipment(eq[num].equipmentKey);
			}
		} else if( rnd < 0.05 * sp ) { //lost a cargo in every 100 sec
			var id = r.$TowbarShips.indexOf(r.$TowbarShip);
			if( id > -1 ) {
				var prevlost = 0;
				if(r.$TowbarLostCargo[id] > 0) prevlost = r.$TowbarLostCargo[id];
				if( prevlost < Math.min(20, r.$TowbarShip.cargoSpaceCapacity / 6 )) { //lost max.20t
					if( Math.random() > 0.2 ) {
						player.consoleMessage("Cargo flying from the towed ship", 5);
						r.$TowbarShip.spawnOne("cargopod");
					} else player.consoleMessage("1t Cargo destroyed in the towed ship", 5);
					r.$TowbarLostCargo[id]++;
				}
			}
		} else if( rnd < 0.1 * sp ) { //lost an alloy in every 20 sec
			var id = r.$TowbarShips.indexOf(r.$TowbarShip);
			if( id > -1 ) {
				var prevlost = 0;
				if(r.$TowbarLostAlloys[id] > 0) prevlost = r.$TowbarLostAlloys[id];
				if( prevlost < r.$Towbar_AlloysFromShip( r.$TowbarShip ) / 1.5 ) { //lost max. 75t
					if( Math.random() > 0.3 ) {
						player.consoleMessage("Alloy flying from the towed ship", 5);
						r.$TowbarShip.spawnOne("alloy");//fragmented from the ship
					} else player.consoleMessage("1t Alloy destroyed in the towed ship", 5);
					r.$TowbarLostAlloys[id]++;
				}
			}
		}
	}
}

this.$Towbar_Mining = function() {
	var ws = worldScripts["towbar"];
	var ship = ws.$TowbarShip;
	if( player.ship.equipmentStatus("EQ_TOWBAR") == "EQUIPMENT_OK"
		&& player.ship.equipmentStatus("EQ_SHIPMINER") == "EQUIPMENT_OK"
		&& ship && ship.isValid
		&& (!ship.script || !ship.script.$TowbarMission ) ) { //skip mission ship
		var id = ws.$TowbarShips.indexOf(ship);
		if( id == -1 ) return; //should not happen
		if( (!ship.script || ship.script.$TowbarMinedShip != 1 )
			&& ws.$TowbarMiningC[id] >= 0 ) { // set to -1 after finished ship mining
			if( ws.$TowbarMiner > 0 ) {
				if( ws.$TowbarMiningC[id] == 0 ) {
					ws.$TowbarMiningC[id] = 1; //start mining, checked in $Towbar_Timed()
					if( ws.$TowbarMiner == 1 && player.ship.cargoSpaceAvailable > 0
						&& (!ship.script || ship.script.$TowbarEmptyShip != 1 ) )
						player.consoleMessage("Cargo transfer started", 10);
					else if( ws.$TowbarMiner == 2 && player.ship.cargoSpaceAvailable > 0 )
						player.consoleMessage("Ship mining started", 10);
					else if( ws.$TowbarMiner == 3 )
						player.consoleMessage("Ship salvaging started", 5);
				} else {
					if( ws.$TowbarMiner == 1 && ws.$TowbarLostCargo[id] >= 100000 ) {
						player.consoleMessage("Cargo transfer already finished, "
							+"full mining need to prime Towbar and press mode", 10);
					} else {
						if( ws.$TowbarMiner == 1
							&& (!ship.script || ship.script.$TowbarEmptyShip != 1 ) ) {
							if( player.ship.cargoSpaceAvailable < 1 )
								player.consoleMessage("Your cargo store is full", 5);
							else player.consoleMessage("Cargo transferring...", 5);
						} else if( ws.$TowbarMiner == 2 && player.ship.cargoSpaceAvailable > 0 )
							player.consoleMessage("Ship mining...", 5);
						else if( ws.$TowbarMiner == 3 )
							player.consoleMessage("Ship salvaging...", 5);
					}
				}
			} else player.consoleMessage("Ship miner offline, prime Towbar and press mode to turn it on", 10);
		} else if( ws.$TowbarMiner > 0 )
			player.consoleMessage("Ship mining already finished", 5);
	}
}

this.$Towbar_Name = function( ship ) { //Detectors OXP support
	var name = ship.displayName;
	if( ship.script && ship.script.$Detectors_Origname )
		name = ship.script.$Detectors_Origname;
	if( name.substring(0, 6) == "Empty " ) name = name.substring(6); //without "Empty " prefix
	return( name );
}

this.$Towbar_ParcelAdd = function() {
	if( system && !system.isInterstellarSpace && system.mainStation && player.ship ) {
		var rnd2 = Math.random();
		var rn = randomName();
//		var rn2 = randomName();
//		if( rnd2 < 0.3 ) rn2 = rn; //within family, will be cheap due to family discount prices
		if( !this.$TowbarParcels ) this.$TowbarParcels = [];
		var len = this.$TowbarParcels.length / this.$TowbarParcelFields + 1; ///
		var start = system.ID; //current system
		var pname = rn+"'s "+expandMissionText("TowbarParcel")+" ("+len
			+". find) from "+System.systemNameForID(start);//+" to "+rn+" "+rn2;
			//+" from "+randomName()+" "+rn+" to "+randomName()+" "+rn2; - too long in F5F5
		var dest = Math.floor( rnd2 * 256 ); //any system
		var dist = System.infoForSystem(galaxyNumber, system.ID).distanceToSystem(
			System.infoForSystem(galaxyNumber, dest));
		for( var i = 0; i < 2; i++ ) { //choose the nearest from 3 random system
			var dest2 = Math.floor( ( i + rnd2 ) * 128 ); //another systems
			var dist2 = System.infoForSystem(galaxyNumber, system.ID).distanceToSystem(
				System.infoForSystem(galaxyNumber, dest2));
			if( dest == start || ( dist2 < dist && dest2 != start ) ) { dest=dest2; dist=dist2; }
		}
		var dname = System.systemNameForID(dest);
		var fee = Math.ceil( ( Math.ceil( 10 * rnd2 * dist ) + 100 )
			* Math.max( 1, player.parcelReputation * 0.5 ) / 100 ) * 100;
		var ahour = Math.ceil( ( 10 - 5 * rnd2 ) * dist ); //time to arrive
		var atime = clock.seconds + Math.min( 3600 * ahour, this.$TowbarParcelMaxTime );
//		if( player.ship.addParcel( pname, start, dest, atime, fee ) ) {
			var msg = "Parcel to "+dname+": "+Math.ceil(dist)+"LY in "
				+Math.ceil(ahour)+" hours for "+formatCredits( fee, false, true );
			this.$TowbarParcels.push( pname );
			this.$TowbarParcels.push( start ); //2. item in array is start system ID
			this.$TowbarParcels.push( dest ); //3. item in array is destination system ID
			this.$TowbarParcels.push( atime ); //4. item in array is arrive time in clock.seconds
			this.$TowbarParcels.push( fee ); //5. item in array is payout
			this.$TowbarParcels.push( galaxyNumber ); //6. item in array is galaxy
			log("Towbar", pname+": "+msg);
			player.commsMessage(msg, 10);
			mission.markSystem( { system: dest, name: pname, markerColor: "greenColor",
					   markerScale: 2, markerShape: "MARKER_DIAMOND" } );
		}
//	}
}

this.$Towbar_ParcelAddUrgent = function() {
	if( system && !system.isInterstellarSpace && system.mainStation && player.ship ) {
		var rnd2 = Math.random();
		var rn = randomName();
//		var rn2 = randomName();
//		if( rnd2 < 0.2 ) rn2 = rn; //within family, will be cheap due to family discount prices
		if( !this.$TowbarParcels ) this.$TowbarParcels = [];
		var len = this.$TowbarParcels.length / this.$TowbarParcelFields + 1; ///
		var start = system.ID; //current system
		var pname = "urgently "+rn+"'s "+expandMissionText("TowbarUrgentParcel")+" ("+len+". find)";
			//len+". urgent parcel of "+rn;//+" from "+System.systemNameForID(start);
			//+" from "+randomName()+" "+rn+" to "+randomName()+" "+rn2; - too long in F5F5
		var dest = system.ID; //current system
		var dist = Math.ceil( player.ship.position.distanceTo( system.mainStation.position )/1000 );
		var fee = Math.ceil( 5 * rnd2 ) * 100 * Math.max(1, player.parcelReputation * 0.5 );
		var aminute = Math.ceil( 1 + 9 * ( 1 - rnd2 ) ); //time to arrive: 2-10min
		var atime = clock.seconds + Math.min( 60 * aminute, this.$TowbarParcelMaxTime );
//		if( player.ship.addParcel(pname, start, dest, atime, fee) ) {
			var msg = "Urgent parcel to "+system.name+" station: "+dist+"km in "
				+aminute+" minutes for "+formatCredits(fee, false, true);
			this.$TowbarParcels.push( pname );
			this.$TowbarParcels.push( start ); //2. item in array is start system ID
			this.$TowbarParcels.push( dest ); //3. item in array is destination system ID
			this.$TowbarParcels.push( atime ); //4. item in array is arrive time in clock.seconds
			this.$TowbarParcels.push( fee ); //5. item in array is payout
			this.$TowbarParcels.push( galaxyNumber ); //6. item in array is galaxy
			log("Towbar", pname+": "+msg);
			player.commsMessage(msg, 10);
			mission.markSystem( { system: dest, name: pname, markerColor: "greenColor",
					   markerScale: 2, markerShape: "MARKER_DIAMOND" } );
//		}
	}
}

this.$Towbar_ParcelLog = function() {
	if( this.$TowbarDebug ) {
		var rep = player.parcelReputation;
		if( player.parcelReputationPrecise ) rep = player.parcelReputationPrecise; //need Oolite v1.79
		log("Towbar", "clock.seconds:"+clock.seconds+" parcelReputation:"+rep);
		var p = this.$TowbarParcels;
		for( var i = 0; i < p.length; i += this.$TowbarParcelFields ) {
			var valid = "expired at";
			if( p[i+3] > clock.seconds ) valid = "valid until";
			else if( p[i+4] < 0 ) valid = "delivered at";
			log("Towbar", p[i]+" "+valid+" "+p[i+3]+" "+" from "+System.systemNameForID(p[i+1])
				+" to "+System.systemNameForID(p[i+2])+" for "+formatCredits(Math.abs(p[i+4]), false, true));
		}
	}
}

this.$Towbar_ParcelRemoveExpired = function( startup ) { //search valid parcels from ParcelMin
	var p = this.$TowbarParcels;
	for( var i = this.$TowbarParcelMin; i < p.length; i += this.$TowbarParcelFields ) {
		if( p[i+3] < clock.seconds && p[i+4] > 0 ) { //atime is in past and still has vaule
			p[i+4] = 0; //no fee anymore, flag to expired (avoid repeated expired message)
			var pname = p[i];
			if( !startup ) {
				var ename = pname;
				if( ename.indexOf("urgently") > -1 ) ename = ename.substr(9);//cut first word+space
				player.consoleMessage( "Expired "+ename, 10 );
			}
			if( this.$TowbarDebug ) log("Towbar", pname+" is expired at "+p[i+3]);
			if( p[i+5] == galaxyNumber ) {
				player.ship.removeParcel( pname ); //needed if manifest screen shown when expire
				mission.unmarkSystem( { system: p[i+2], name: pname, markerColor: "",
					   markerScale: 2, markerShape: "" } );
			}
			if( i == this.$TowbarParcelMin + this.$TowbarParcelFields )
				this.$TowbarParcelMin = i; //increase until continuosly expired
		}
	}
	if( this.$TowbarDebug ) {
		var rep = player.parcelReputation;
		if( player.parcelReputationPrecise ) rep = player.parcelReputationPrecise; //need Oolite v1.79
		log("Towbar", "clock.seconds:"+clock.seconds+" parcelReputation:"+rep
			+" Parcels:"+(p.length/this.$TowbarParcelFields)
			+" ParcelMin:"+(this.$TowbarParcelMin/this.$TowbarParcelFields));
	}
}

this.$Towbar_ParcelRemoveExpired_old = function( startup, addtime ) { //search valid parcels from ParcelMin
	return;//renamed to _old and exit here for sure
	var rep = player.parcelReputation;
	if( player.parcelReputationPrecise ) rep = player.parcelReputationPrecise; //need Oolite v1.79
	this.$TowbarParcelReputation = rep; //save for Towbar_ParcelReputationRestore
	log("Towbar", "clock.seconds:"+clock.seconds+" parcelReputation:"+rep);
	var p = this.$TowbarParcels;
	for( var i = this.$TowbarParcelMin; i < p.length; i += this.$TowbarParcelFields ) {
		if( p[i+3] < clock.seconds + addtime ) { //atime is in past
			var pname = p[i];
			if( !startup && !player.ship.addParcel( pname, 1, 1, clock.seconds + 100, 1 ) ) { //old version
				var ename = pname;
				if( ename.indexOf("urgently") > -1 ) ename = ename.substr(9);//cut first word+space
				player.consoleMessage( "Expired "+ename, 10 );
			}
			this.$TowbarParcelExpired++; //how many reputation tenth need to be restored after hyperjump
			if( this.$TowbarDebug ) log("Towbar", pname+" is expired at "+p[i+3]);
			player.ship.removeParcel( pname );
			mission.unmarkSystem( { system: p[i+2], name: pname, markerColor: "",
					   markerScale: 2, markerShape: "" } );
			if( i == this.$TowbarParcelMin + this.$TowbarParcelFields )
				this.$TowbarParcelMin = i; //increase until continuosly expired
		}
	}
	var rep = player.parcelReputation;
	if( player.parcelReputationPrecise ) rep = player.parcelReputationPrecise; //need Oolite v1.79
	if( this.$TowbarDebug ) log("Towbar", "clock.seconds:"+clock.seconds+" parcelReputation:"+rep
		+" Parcels:"+(p.length/this.$TowbarParcelFields)
		+" ParcelMin:"+(this.$TowbarParcelMin/this.$TowbarParcelFields));
//	this.$Towbar_ParcelReputationRestore();//here is not working, need to call from elsewhere a few times later
}

this.$Towbar_ParcelReputationRestore_old = function() {  //old inaccurate version, can not follow sudden reputation changes
	return;//renamed to _old and exit here for sure
	if( player.parcelReputationPrecise ) { //from Oolite v1.79
		while( player.parcelReputationPrecise < this.$TowbarParcelReputation
			&& this.$TowbarParcelExpired-- > 0 ) {
			player.increaseParcelReputation();
			if( this.$TowbarDebug ) log("Towbar", "ParcelReputationRestore:"+player.parcelReputationPrecise
				+", "+this.$TowbarParcelExpired+" expire left");
		}
	} else { //inaccurate method for Oolite v1.77 due to no tenth available, disabled
		while( //player.parcelReputation <= this.$TowbarParcelReputation &&
			this.$TowbarParcelExpired-- > 0 ) {
//			player.increaseParcelReputation();
//			if( player.parcelReputation > this.$TowbarParcelReputation ) //too many
//				player.decreaseParcelReputation();
			if( this.$TowbarDebug ) log("Towbar", "ParcelReputationRestore:"+player.parcelReputation
				+", "+this.$TowbarParcelExpired+" expire left");
		}
	}
	this.$TowbarParcelReputation = -1000; //reset
}

this.$Towbar_ParcelTimed = function() { //check and remove from manifest screen in every 10 minutes
//	this.$Towbar_ParcelReputationRestore();
	this.$Towbar_ParcelRemoveExpired( false );
}

this.$Towbar_Payout = function(ship) { //pay reward for the salvaged ship
	var r = worldScripts["towbar"];
	if( ship && ship.isValid && (!ship.script || !ship.script.$TowbarMission)) { //exclude mission ships
		var name = r.$Towbar_Name( ship );
		if( r.$TowbarDebug ) log("Towbar", name+" "+
			ship.entityPersonality+" mass:"+Math.round(ship.mass/1000)
				+"t maxCargo:"+ship.cargoSpaceCapacity+"t ");//debug
		var t = r.$Towbar_AlloysFromShip( ship );
		var id = r.$TowbarShips.indexOf(ship);
		if( id > -1 ) {
			t -= r.$TowbarLostAlloys[id];
			r.$TowbarLostAlloys[id] = 0;
		}
		if( t < 1 ) t = 1; //for sure
		var station = system.mainStation;
		if( player.ship.docked ) station = player.ship.dockedStation;
		var ac = station.market.alloys.price / 10; //32.8; //alloy cost
		var cr = Math.round( ac * t / 10 ) * 10;
		if( r.$TowbarDebug ) log("Towbar", ship.displayName+" "+
			ship.entityPersonality+" "+t+"t alloys "+cr+" credits");//debug
//		station.setMarketQuantity( "alloys", Math.min(127, station.market.alloys.quantity + t ) );//sold to a dealer
		player.credits += cr;
		var t = r.$Towbar_Computers( ship );
		if( t > 0 ) { //can be negative if lost many cargo container durig towing
			var ccr = Math.round( t * station.market.computers.price / 100 ) * 10; ///cargo cost
			player.credits += ccr;
			cr += ccr;
			if( r.$TowbarDebug ) log("Towbar", ship.displayName+" "+
				ship.entityPersonality+" "+t+"t computers "+ccr+" credits");//debug
		}
		
		var mined = false;
		if( ship.script && ship.script.$TowbarMinedShip == 1 ) mined = true;//avoid rep. again after mined
		else if( missionVariables.$TowbarShips > 0 ) missionVariables.$TowbarShips++; //salvager reputation
			else missionVariables.$TowbarShips = 1;
		var name = r.$Towbar_Name( ship );
		var msg = "You get "+cr+" credits for the salvaged hull of the "+name
			+", your "+missionVariables.$TowbarShips+". ship.";
		var tc = cr; //total credits

		//cargo and valuables
		var msg2 = "\n";//skip the line if no cargo
		var cargo = r.$Towbar_Cargo( ship );
		r.$TowbarLostCargo[id] = 100000; //more than any ship can carry including bases to flag it is empty
		var cr1 = 0;
		var cr2 = 0;
		var ccr = 0;
		if( player.ship.docked ) {
			var mc = worldScripts["mo-commodity_markets"]; //different buy and sell prices
			if( cargo.t > 0 ) { //can be negative if lost many cargo container durig towing
				var pr = station.market[cargo.name].price;
				if(mc && mc.$priceFactor) {
					var id = mc.$commodities.indexOf(cargo.name);
					if( id > -1 ) pr /= mc.$priceFactor[id]; //2-5% less
				}
				if( r.$TowbarDebug && mc ) log("Towbar", "id:"+id+" pr:"+pr+" mcpf:"+mc.$priceFactor);
				var sq = station.market[cargo.name].quantity;
				var q = cargo.t;
				if( q + sq > 127 ) q = 127 - sq;
				cr1 = Math.round( q * pr / 10 ); //cargo cost
				station.setMarketQuantity(cargo.name, Math.min(127, station.market[cargo.name].quantity + q));
				if( !r.$TowbarLimit127t && cargo.t > q ) {
					cr1 += Math.round((cargo.t-q) * pr / 20 ); //half cost
				} else cargo.t = q; //remains discarded
				cr += cr1;
			}
			if( cargo.v > 0 ) {
				var pr = station.market[cargo.vname].price;
				if(mc && mc.$priceFactor) {
					var id = mc.$commodities.indexOf(cargo.vname);
					if( id > -1 ) pr /= mc.$priceFactor[id]; //2-5% less
				}
				var sq = station.market[cargo.vname].quantity;
				var q = cargo.v;
				if( q + sq > 127 ) q = 127 - sq;
				cr2 = Math.round( q * pr / 10 ); //valuables cost
				station.setMarketQuantity(cargo.vname, Math.min(127, station.market[cargo.vname].quantity+q));
				if( !r.$TowbarLimit127t && cargo.v > q ) {
					cr2 += Math.round((cargo.v-q) * pr / 20 ); //half cost
				} else cargo.v = q; //remains discarded
				cr += cr2;
			}
			ccr = Math.round((cr1 + cr2)/10)*10; //store cargo credits to deduct from total to get ship value later
			if( cargo.t > 0 || cargo.v > 0 ) {
				player.credits += ccr;
				if(ccr > 0) {
					msg2 += "You get "+ccr+" credits for ";
					if( cargo.t > 0 ) msg2 += cargo.t+"t "+cargo.name;
					if( cargo.t > 0 && cargo.v > 0 ) msg2 += " and ";
					if( cargo.v > 0 ) msg2 += cargo.v+cargo.vunit+" "+cargo.vname;
					msg2 +=".";
				}
				tc += ccr;
				if( r.$TowbarDebug ) {
					var vs = "";
					if( cargo.v > 0 ) vs = cargo.v+cargo.vunit+" "+cargo.vname+" ";
					log("Towbar", ship.displayName+" "+ship.entityPersonality+" "+cargo.t+"t "
						+cargo.name+" "+vs+ccr+" credits, "+tc+" credits in total");
				}
			} else msg2 += "There are no cargo in "+name+".";
		} else { //Tug Drone drop out all cargo
			var cdname = "cargo";
			if( cargo.t > 0 ) {
				if(system && system.mainStation && system.mainStation.market)
					//cdname = system.mainStation.market[cargo.name].displayName;
					cdname = cargo.name;
				else cdname = cargo.name;
				for( var j = 0; j < cargo.t; j += 20 ) {
					var lost = Math.min(20, cargo.t - j); //max. 20t in one large cargo box
					var a = null;
					if( lost > 1 ) a = ship.spawnOne("more-cargo");
					if( a && a.isValid ) {
						if( !a.script ) a.script = "oolite-default-ship-script.js";
						if( a.script ) {
							a.script.$cargo = lost;
							a.script.$cargoname = cargo.name;
						}
						a.displayName = lost+"t "+cdname;
					} else {
						var p = ship.spawn("cargopod", Math.min(64, lost) );
						if( p ) for(var k=0; k<p.length; k++) {
							if( p[k] && p[k].isValid ) {
								p[k].setCargo(cargo.name);
								p[k].displayName = "1t "+cdname;
								p[k].bounty = 0;//to avoid * mark with detectors oxp
							}
						}
					}
					player.consoleMessage(lost+"t "+cdname+" flying from "
						+r.$Towbar_Name( ship ), 5);
				}
			}
			if( cargo.v > 0 ) {
				if(system && system.mainStation && system.mainStation.market)
					//cdname = system.mainStation.market[cargo.vname].displayName;
					cdname = cargo.vname;
				else cdname = cargo.vname;
				var a = null;
				if( cargo.v > 1 ) {
					if(cargo.vunit == "t") a = ship.spawnOne("more-cargo");
					else a = ship.spawnOne("small-cargo"); //some kg gold can fit into a small pod
				}
				if( a && a.isValid ) {
					if( !a.script ) a.script = "oolite-default-ship-script.js";
					if( a.script ) {
						a.script.$cargo = cargo.v;
						a.script.$cargounit = cargo.vunit;
						a.script.$cargoname = cdname;
					}
					a.displayName = cargo.v+cargo.vunit+" "+cdname;
				} else  {
					var p = ship.spawn("cargopod", Math.min(64, cargo.v) );
					if( p ) for(var k=0; k<p.length; k++) {
						if( p[k] && p[k].isValid ) {
							p[k].setCargo(cargo.vname);
							p[k].displayName = "1"+cargo.vunit+" "+cdname;
							p[k].bounty = 0;//to avoid * mark with detectors oxp
						}
					}
				}
				player.consoleMessage(cargo.v+cargo.vunit+" "+cdname+" flying from "
					+r.$Towbar_Name( ship ), 5);
			}
			if( cargo.t > 0 || cargo.v > 0 ) {
				player.credits += ccr;
				msg2 += "Tug Drone dropped out ";
				if( cargo.t > 0 ) msg2 += cargo.t+"t "+cargo.name;
				if( cargo.t > 0 && cargo.v > 0 ) msg2 += " and ";
				if( cargo.v > 0 ) msg2 += cargo.v+cargo.vunit+" "+cargo.vname;
				msg2 +=" from "+name+".";
			} else msg2 += "Tug Drone is not found any cargo in "+name+".";
		}

		var i=-1;//index in the eq array
//		ship.awardEquipment("EQ_MILITARY_JAMMER");//debug
		var eq = ship.equipment; //local variable, faster
		var d = "";
		var eqs = 0;
		var hiddeneq = 0;
		var hiddeneqcr = 0;
		var lasthiddeneqcr = 0;
		var lasthiddeneqname = "";
		var maxeq = 5; //5 equipment line can fit into the main part of the screen
 		while( eq[++i] ) {
			if( eq[i] && eq[i].equipmentKey != "EQ_MILITARY_JAMMER") { //skip MASCM due to do not fit into one line
				if( r.$TowbarDebug )
					log("Towbar", ship.displayName+" "+
						ship.entityPersonality+" "+(i+1)+". "+eq[i].name);
				d = "";
				var p = eq[i].price * 0.01; //max. 10% of the original cost in credits (not in credits*10)
				var rnd = Math.random();
				if( p > 300 ) {
					rnd = rnd * ( 300 / p ); //costly eqs always damaged
					cr = Math.round(Math.max(50, Math.round( p * rnd ))/10)*10; //but at least 50cr
				} else cr = Math.round(Math.max(10, Math.round( p * rnd ))/10)*10; //at least 10cr
				if( rnd < 0.5 ) d = " the damaged";
				player.credits += cr;
				tc += cr;
				if( cr > 0 ) { // exclude 0 to fit into the screen with ShipVersion OXP
					if( eqs < maxeq ) {
						if(cr > 0) msg2 += "\nYou get "+cr+" credits for"
							+d+" "+eq[i].name+".";
						eqs++; //listed eqs only
					} else { //hidden eq
						hiddeneq++;
						hiddeneqcr += cr;
						lasthiddeneqcr = cr;
						lasthiddeneqname = eq[i].name;
					}
				}
			}
		}
		if( hiddeneq > 1 ) {
			if(hiddeneqcr > 0)
				msg2 += "\nYou get "+hiddeneqcr+" credits for "+hiddeneq+" more equipments.";
		} else if( hiddeneq == 1 ) {
			if(lasthiddeneqcr > 0)
				msg2 += "\nYou get "+lasthiddeneqcr+" credits for"+d+" "+lasthiddeneqname+".";
		}
		if( eqs == 0 ) msg2 += "\nThere are no equipments in "+name+".";
		eqs++; //need in all cases including no hidden but more than 0 eqs

		var w = ship.forwardWeapon;
		msg2 += "\n";
		if( w && !mined ) {
			cr = Math.round((w.price/10*(0.1+Math.random()*0.1))/10)*10; //min. 10%, max. 20% of the original cost
			player.credits += cr;
			tc += cr;
			if(cr > 0) msg2 += "You get "+cr+" credits for "+w.name+".";
		} else if(!ship.aftWeapon || mined) msg2 += "There are no laser weapons in "+name+".";
		
		var w2 = ship.aftWeapon;
		msg2 += "\n";
		if( w2 && !mined ) {
			if( w2 != w ) //keep the cost if aft weapon is the same
				cr = Math.round((w2.price/10*(0.1+Math.random()*0.1))/10)*10; //min. 10%, max 20%
			player.credits += cr;
			tc += cr;
			if(cr > 0) msg2 += "You get "+cr+" credits for aft "+w2.name+".";
		}
		var shipcr = tc - ccr; //tax payed from ship hull and equipments only, not from cargo
		
		while( eqs++ < maxeq ) msg2 += "\n"; //put down the total lines to the bottom window if a few eqs listed only
		msg += msg2+"\n";

		var place = "";
		var tax = 0;
		var fee = 0;
		if( player.ship.docked && station.isMainStation ) {
			var lcr = 0;
			if( missionVariables.$TowbarLicense > 0 ) lcr = missionVariables.$TowbarLicense;
			var ldate = 0;
			if( missionVariables.$TowbarLicenseDate > 0 ) ldate = missionVariables.$TowbarLicenseDate;
			if( ldate > 0 && clock.days > ldate + 365 && lcr >= r.$TowbarLicenseFee ) { //out of date
				lcr = 0; //must get new license in every year
				missionVariables.$TowbarLicense = 0;
				msg += "\nYour license is outdated, renew required.";
			}
			var taxp = 0.05 * system.government; //0:Anarchy (0%) - 7:Corporate State (35%)
			var msgtax = "";
			if( system.government > 0 ) {
				tax = Math.ceil(shipcr * taxp);
				msgtax = "\nYou pay "+tax+" credits as "+Math.round(taxp*100)+"% tax in "
					+system.governmentDescription+".";
			} else msgtax = "\nNo tax in Anarchy.";
			if( lcr >= r.$TowbarLicenseFee ) {
				place = "with Salvager License";
				player.credits -= tax;
				msg += msgtax;
			} else {
				place = "without license";
				if( lcr + shipcr >= r.$TowbarLicenseFee ) {
					//surely the first time due to next time handled before
					missionVariables.$TowbarLicense = r.$TowbarLicenseFee;
					msg += "\nCongratulations, you get the Salvager License for a year!";
					fee = r.$TowbarLicenseFee - lcr;
					if( fee < tax ) {
						fee = tax;//pay the larger
 						msg += "\nPayed "+fee+" credits as tax.";
					} else msg += "\nPayed "+fee+" credits, next time you will pay tax only.";
					player.credits -= fee;
					missionVariables.$TowbarLicenseDate = clock.days; //save date
				} else { //get the full shipcr into the licence fee
					missionVariables.$TowbarLicense = lcr + shipcr;
					player.credits -= shipcr;
					fee = shipcr;
					msg += "\nPut "+shipcr+" credits into your license pool.\n"
						+ ( r.$TowbarLicenseFee - missionVariables.$TowbarLicense )
						+" credits left to get the Salvager License.";
				}
			}
		} else {
			if( player.ship.docked && ( station.name == "Salvage Gang" 
				|| station.name == "Deep Space Dredger" ) ) {
				msg += "\n"+station.name+" do not ask commission over the low alloy price.";
				place = "to the "+station.name;
			} else { //other stations like Rock Hermit or undocked which means using Tug Drone
				var taxp = 0.5; //50% if not in main station
				tax = Math.ceil(shipcr  * taxp);
				player.credits -= tax;
				msg += "\nYou payed "+tax+" credits as "+Math.round(taxp*100)+"% commission.";
				if( player.ship.docked ) place = "away from the main station";
				else place = "via Tug Drone";
			}
		}

		if( fee > 0 ) tc -= fee;
		else tc -= tax;
		msg += "\nTotal "+tc+" credits";
		if( tc >= 5000 ) msg += ", excellent!";
		else if( tc >= 3000 ) msg += ", well done!";
		else if( tc >= 1000 ) msg += ", nice work.";
		else {
			msg += ". You can get more ";
			if( lcr < r.$TowbarLicenseFee ) msg += "with license.";
			else msg += "for a larger ship.";
		}
		
		if( missionVariables.$TowbarCash > 0 ) missionVariables.$TowbarCash += tc;
		else missionVariables.$TowbarCash = tc;
		
		r.$TowbarLostAlloys[id] = 0;
		r.$TowbarLostCargo[id] = 0;
		r.$TowbarLostComps[id] = 0;
		r.$TowbarMiningC[id] = 0;
		ship.remove(true);
		
		var model = null;
		if( player.ship.docked ) {
		//need to check, else: ReportJSError (OOJavaScriptEngine.m:198): 
		//----- JavaScript warning (tug drone 0.9): Mission.runScreen: model cannot be displayed while in flight.
			model = "["+ship.dataKey+"]";
			r.$TowbarShip = null;
			if( r.$TowbarInjectors == 1 &&
				player.ship.equipmentStatus("EQ_FUEL_INJECTION") == "EQUIPMENT_DAMAGED" ) {
				player.ship.setEquipmentStatus("EQ_FUEL_INJECTION", "EQUIPMENT_OK" );
				r.$TowbarInjectors = 0;
			}
		}
		
		r.$Towbar_RunScreen(place, msg, model); //show sell salvage screen
	}
}

this.$Towbar_RefundEQ = function( eq ) {
	if( player.ship.equipment.indexOf( eq ) ) {
		if( player.ship.equipmentStatus( eq ) == "EQUIPMENT_DAMAGED" )
			player.consoleMessage("Need repair first"); //do not get back full price for a damaged eq
		else if( player.ship.equipmentStatus( eq ) == "EQUIPMENT_OK" ) {
			player.ship.removeEquipment( eq );
			clock.addSeconds ("3600");
			var refund = EquipmentInfo.infoForKey( eq ).price / 10;///
			player.credits += refund;
			player.consoleMessage("Refunded "+refund+" credits for "+EquipmentInfo.infoForKey( eq ).name);
			return( refund );
		}
	}
	return( 0 );
}

this.$Towbar_Release = function() { //release towed ship
//	if( this.$TowbarDebug ) log("Towbar",system.addShips("anarchies-salvage-gang",1,player.ship.position, 15000));
	if( this.$TowbarShip && this.$TowbarShip.isValid ) {
//		this.$TowbarShip.position = player.ship.position.add( player.ship.vectorForward.multiply( -300 ) );
		var s = "";
		var id = this.$TowbarShips.indexOf(this.$TowbarShip);
		if( id > -1 && this.$TowbarMiningC[ id ] > 0 ) {
			var assist = 1; //need set to 1 exactly if no miner assistant and 2 if exist
			if( player.ship.equipmentStatus("EQ_SHIPMINERASSISTANT") == "EQUIPMENT_OK" ) assist = 2;
			if( this.$TowbarMiner > 1 ) s = ", mining suspended";
			else if( this.$TowbarMiner == 1 && this.$TowbarMiningC[ id ] < this.$TowbarMining1 / assist
				&& this.$TowbarLostCargo[id] < 100000 ) ///
				s = ", cargo transfer suspended";
		}
 		player.consoleMessage( "Towbar released "+this.$Towbar_Name( this.$TowbarShip )+s, 10);
		if( this.$TowbarInjectors == 1 &&
			player.ship.equipmentStatus("EQ_FUEL_INJECTION") == "EQUIPMENT_DAMAGED" ) {
			player.ship.setEquipmentStatus("EQ_FUEL_INJECTION", "EQUIPMENT_OK" );
			this.$TowbarInjectors = 0;
 			player.consoleMessage( "Injectors online", 5);
		}
	}
	this.$TowbarShip = null; //release towed ship
	this.$TowbarPrevOri = null; //need to avoid infinite relock after anchor release
	if( !this.$TowbarFixed ) this.$Towbar_VClear();
}

this.$Towbar_RunScreen = function(place, msg, model) { //show sell salvage screen
	if( guiScreen != "GUI_SCREEN_MISSION" ) { //need to avoid overwrite possible other mission screens
		mission.runScreen({
			title: "Sell salvage "+place,
			message: msg,
			choices: {"_0" : "OK"},
			model: model,
			spinModel:false,
			background: worldScripts.towbar.$TowbarBG
		},function(choice) {
			var r = worldScripts.towbar;
			r.$TowbarSound4.play();//sell sound
			r.$Towbar_UpdateReputation(false);
			if( isValidFrameCallback( r.$TowbarFCB2 ) )
				removeFrameCallback( r.$TowbarFCB2 );
		});//no function
		if( player.ship.docked ) { //mission.displayModel nonexistent during flight
			var r = worldScripts.towbar;
			var shift = Math.max( mission.displayModel.collisionRadius * 0.9,  //fix for adder
						0.5 * mission.displayModel.position.z
						- Math.max( player.ship.boundingBox.x, player.ship.boundingBox.y ) );
			if( r.$TowbarDebug ) log("Towbar", mission.displayModel.position+ " shift:"+ shift);
			// camera is at [0,0,0] facing [1,0,0,0]
			mission.displayModel.position =Vector3D( mission.displayModel.position.x + shift, //shift to right side
								mission.displayModel.position.y,
								mission.displayModel.position.z + 10);
			mission.displayModel.orientation = //fix flashing shaders in 1.81
				mission.displayModel.orientation.rotateX( 0.01 );

			if( !isValidFrameCallback( r.$TowbarFCB2 ) )
				r.$TowbarFCB2 = addFrameCallback( r.$Towbar_FCB2 );
		}
	} else worldScripts.towbar.$TowbarRunScreens.push({place:place, msg:msg, model:model}); //save to call later from FCB
}

this.$Towbar_ScannerColor = function(ship) {
	this.$Towbar_ScannerColor2(ship, false);
}

this.$Towbar_ScannerColor2 = function(ship, playertarget) {
	if( playertarget || ship && ship == player.ship.target ) {
		player.ship.removeEquipment("EQ_DTAEMP"); //must before isDerelict check to remove when retarget to piloted
		player.ship.removeEquipment("EQ_DTAMIN");
	}
	if( !ship || !ship.isValid || !ship.isDerelict ) return;
	if( ship.script && ship.script.$TowbarMinedShip == 1 ) {
		ship.scannerDisplayColor1 = "0 0 0.7"; //darker blue if mined
		if( playertarget || ship == player.ship.target ) {
			player.ship.awardEquipment("EQ_DTAMIN");
			player.ship.removeEquipment("EQ_DTAEMP");
		}
	} else {
		var id = this.$TowbarShips.indexOf(ship);
//		log("Towbar", "id:"+id+" e:"+ship.script.$TowbarEmptyShip+" lostc:"+this.$TowbarLostCargo[id]);//debug
		if( ship.script && ship.script.$TowbarEmptyShip == 1
			|| id > -1 && this.$TowbarLostCargo[id] >= 100000 ) {
			ship.scannerDisplayColor1 = "0 0 1"; //blue if empty (cargo transferred)
			if( playertarget || ship == player.ship.target ) {
				player.ship.awardEquipment("EQ_DTAEMP");
				player.ship.removeEquipment("EQ_DTAMIN");
			}
		} else {
			ship.scannerDisplayColor1 = "0.1 0.1 1"; //a bit lighter blue if unmined derelict
		}
	}
}

this.$Towbar_SetTimedBombToDerelict = function( ship ) {
	if( ship && ship.isValid && ship.isDerelict ) {
		var e = system.filteredEntities(this, function () { return true; }, ship, 25600);
		var s = null;
		var i = 0;
		for( i in e ) { //NPCs stop attacking derelict ships
			s = e[i];
			if( s && s.isValid && !s.isMissile && s.target == ship && !s.isPlayer )
				s.target = null; //need exclude missiles to do not destroy tug drone
		}
		//set timed bomb
		if( !ship.script ) ship.setScript("oolite-default-ship-script.js");
		if( ship.script ) {
			if( ship.script.$TowbarMission ) ship.script.$Towbar_TimedBombCounter = -1; //no timed bomb
			if( !ship.script.$Towbar_TimedBombCounter ) {
				var sv = Math.max( 1, ( Math.floor( ship.entityPersonality / 328 ) + 1 ) / 10 );
				var rnd = Math.random();
				if( rnd < sv / 50 ) //2-20% chance for timed bomb present depending on ship version
					ship.script.$Towbar_TimedBombCounter = Math.random() * 24;//*5sec=2 minutes max.
				else ship.script.$Towbar_TimedBombCounter = -1; //no timed bomb on board
				ship.script.$Towbar_TimedBombFunction = function() {
					var c = this.$Towbar_TimedBombCounter;
					if( c >= 0 ) {
						this.$Towbar_TimedBombCounter--;
						if( c < 1 ) worldScripts["towbar"]
							.$Towbar_ShipTimedBombExplode(this.ship);
					}
				}
				ship.script.$Towbar_TimedBombTimer = new Timer(ship,
					ship.script.$Towbar_TimedBombFunction, 5, 5);//in every 5 second
			}
		}
	}
}

this.$Towbar_ShipExplode = function() {
	this.$Towbar_ShipExplode2(false);
}

this.$Towbar_ShipExplode2 = function( harmless ) {
	var r = worldScripts["towbar"];
	if( r.$TowbarInjectors == 1 &&
		player.ship.equipmentStatus("EQ_FUEL_INJECTION") == "EQUIPMENT_DAMAGED" ) {
		player.ship.setEquipmentStatus("EQ_FUEL_INJECTION", "EQUIPMENT_OK" );
		r.$TowbarInjectors = 0;
		player.consoleMessage("Injectors online",10);
	}
	r.$Towbar_ShipExplode3(r.$TowbarShip, harmless, true);
	r.$TowbarShip = null;
}

this.$Towbar_ShipExplode3 = function( ship, harmless, towed ) {
	var r = worldScripts["towbar"];
	if(ship && ship.isValid) {
		if( r.$TowbarDebug ) log("towbar", r.$Towbar_Name( ship )+" exploded!");
		if( !harmless ) {
			var t = "";
			if(towed) t = "Towed ";
			else if( ship.isDerelict ) t = "Derelict ";
			player.consoleMessage(t+r.$Towbar_Name( ship )+" exploded!",10);
			var dmg = r.$Towbar_ShipExplodeDamageSize(ship) * ( 0.5 + 0.5 * Math.random() );
			ship.dealEnergyDamage(dmg, 500, 0);
		}
		if(ship && ship.isValid) ship.explode();
	}
	if(towed) r.$TowbarSound3.play();//big bang sound
}

this.$Towbar_ShipExplodeDamageSize = function(ship) {
	return( ship.maxEnergy * 3 / 4 );
}

this.$Towbar_ShipTimedBombExplode = function(ship) {
	worldScripts["towbar"].$Towbar_ShipExplode3(ship, false, false);
}

this.$Towbar_Show = function() {
	if( player.ship.equipmentStatus("EQ_TOWBAR") == "EQUIPMENT_OK" ) {
		var ws = worldScripts["towbar"];
		if( ws.$TowbarShow && !ws.$TowbarV ) {
			var r = system.addVisualEffect("towbar", player.ship.position );
			if( r ) {
				r.scale( 0.03 ); //shrink
				r.scaleZ = r.scaleZ * 160; //make longer, 170=200m aft, 160 due to minus a small fighter length/2
				ws.$TowbarV = r;
			}
		}
	}
}

this.$Towbar_Start = function() { //show and start timer and FCB
	this.$TowbarShip = null; //release towed ship, need if invalid
	if( this.$TowbarFixed ) this.$Towbar_Show();
	if( player.ship.equipmentStatus("EQ_TOWBAR") == "EQUIPMENT_OK" )
		this.$TowbarTimer = new Timer(this, this.$Towbar_Timed, 1, 1);
	if( !isValidFrameCallback( this.$TowbarFCB ) )
		this.$TowbarFCB = addFrameCallback( this.$Towbar_FCB );
}

this.$Towbar_SteerTo = function( otherShip ) { //turn to touch the derelict ship with the towbar then lock on
	worldScripts["towbar"].$Towbar_SetTimedBombToDerelict( otherShip );
	if( player.ship.equipmentStatus("EQ_TOWBAR") == "EQUIPMENT_OK" &&
		otherShip && otherShip.isValid && otherShip.isDerelict &&
		otherShip != worldScripts["towbar"].$TowbarShip ) {
			
//todo: make steering

		//check speed difference
		var diff = player.ship.velocity.subtract( otherShip.velocity ).magnitude();
		if( diff < 10 ) worldScripts["towbar"].$Towbar_Lock( otherShip ); //leave some difference to able to catch in flight
		else player.consoleMessage("Towbar cannot lock: speed difference ("+Math.round(diff)+")");
	}
}

this.$Towbar_Timed = function() {
	var ship = this.$TowbarShip;
	if( ( !ship || !ship.isValid ) && worldScripts["towbar"].$TowbarInjectors == 1 ) {
		//enable injectors if the towed ship died meantime
		worldScripts["towbar"].$TowbarInjectors = 0;
		if( player.ship.equipmentStatus("EQ_FUEL_INJECTION") == "EQUIPMENT_DAMAGED" ) {
			player.ship.setEquipmentStatus("EQ_FUEL_INJECTION", "EQUIPMENT_OK" );
			player.consoleMessage("Injectors online",10);
		}
	}
	
	if( !this.$TowbarV && this.$TowbarShow && (this.$TowbarFixed || this.$TowbarShip) )
		this.$Towbar_Show();//bugfix

	if( player.ship.target && player.ship.target.isValid ) {//set lollipop color of the targeted derelict ship
		this.$Towbar_ScannerColor2(player.ship.target, true); //forced player target
		this.$Towbar_SetTimedBombToDerelict( player.ship.target );
	}
	
	if( this.$TowbarRepairC > 0 ) { //repairing
		this.$TowbarRepairC--;
		if( this.$TowbarRepairC <= 0 ) { //repair done
			player.ship.setEquipmentStatus( "EQ_TOWBAR", "EQUIPMENT_OK" );
			player.consoleMessage("Towbar repaired!", 10);
			this.$TowbarPitchC = 0;
			if( this.$TowbarFixed ) this.$Towbar_Show();
		}
	}

	this.$Towbar_Lose(false); //chance to lose something from the towed ship when move
	
	var assist = 1; //need set to 1 exactly if no miner assistant and 2 if exist
	if( player.ship.equipmentStatus("EQ_SHIPMINERASSISTANT") == "EQUIPMENT_OK" ) assist = 2;
	var id = this.$TowbarShips.indexOf(ship);
//	player.consoleMessage(id+" "+this.$TowbarMiningC[id]);//debug
	if( this.$TowbarMiner == 2 && player.ship.cargoSpaceAvailable < 1 ) { //up to full mode finished
		if( this.$TowbarPrevMiner != this.$TowbarMiner //message only if ship miner mode changed
			&& ship && ship.isValid //and towed ship valid
			&& ( !ship.script || ship.script.$TowbarMinedShip != 1 ) ) //and not finished mining
			player.consoleMessage("Your store is full, salvaging need to prime Towbar and press mode", 10);
	} else if( this.$TowbarMiner == 1 //cargo only mode finished?
		&& ship && ship.isValid //and towed ship valid
		&& ( ship.script && ship.script.$TowbarEmptyShip == 1
			|| id > -1 && this.$TowbarMiningC[id] >= this.$TowbarMining1 / assist - 0.1 ) ) {
		if( id > -1 && this.$TowbarMiningC[id] <= this.$TowbarMining1 / assist + 0.9 ) {
//			message moved before parcels
//			player.consoleMessage("Cargo transfer finished, mining need to prime Towbar and press mode", 10);
			this.$TowbarMiningC[id]++; //to avoid repeated message
		}
	} else if( this.$TowbarMiner > 0 && this.$TowbarMiningC[id] > 0
		&& player.ship.equipmentStatus("EQ_SHIPMINER") == "EQUIPMENT_OK" //mining if miner ok
		&& ship && ship.isValid //and towed ship valid
		&& this.$TowbarPrevSpeed == player.ship.speed //and speed is not changing
		&& (!this.$TowbarPrevOri ||
		player.ship.heading.angleTo( this.$TowbarPrevOri.vectorForward() ) < 0.0005 ) //and not turning
		&& player.ship.speed <= player.ship.maxSpeed ) { //and not injecting nor torus travel nor q-charged
		
		this.$TowbarMiningC[id]++;//must be count all integer number!
		var c = this.$TowbarMiningC[id];
		if( c > this.$TowbarMining1 / assist && this.$TowbarLostCargo[id] < Math.min(120, ship.cargoSpaceCapacity)
			&& player.ship.cargoSpaceAvailable > 0 ) { //there are remaining cargo after transferred
			c = this.$TowbarMiningC[id] = this.$TowbarMining1 / assist; //do another cargo transfer
		}
		var name = this.$Towbar_Name( ship );
		var rnd = Math.random();
		if( c < this.$TowbarMining1 / assist - 0.1 ) { ///do not use equal to avoid rounding problems
			if( rnd < 0.01 ) { //15%
				var cargo = this.$Towbar_Cargo( ship );
				if( cargo.t >= 1 ) {
					cargo.t = Math.max( 1, Math.floor( cargo.t / assist ) ); ///Miner Assist catch the half
					var cdname = cargo.name;
//					if(system && system.mainStation && system.mainStation.market)
//						cdname = system.mainStation.market[cargo.name].displayName;
					if( cargo.t > 1 ) {
						var a = ship.spawnOne("more-cargo");
						if( a && a.isValid ) {
							if( !a.script ) a.script = "oolite-default-ship-script.js";
							if( a.script ) {
								a.script.$cargo = cargo.t;
								a.script.$cargoname = cargo.name;
							}
							a.displayName = cargo.t+"t "+cdname;
						} else {
							var a = this.$TowbarShip.spawn("cargopod", Math.min(64, cargo.t));
							if( a ) for(var i=0; i<a.length; i++) {
								if( a[i] && a[i].isValid ) {
									a[i].setCargo(cargo.name);
									a[i].displayName = "1t "+cdname;
									a[i].bounty = 0;//to avoid * mark with detectors oxp
								}
							}
						}
					} else if( cargo.t > 0 ) {
						var a = this.$TowbarShip.spawnOne("cargopod");
						if( a && a.isValid ) {
							a.setCargo(cargo.name);
							a.displayName = "1t "+cdname;
							a.bounty = 0;//to avoid * mark with detectors oxp
						}
					}
					if( a ) player.consoleMessage(cargo.t+"t "+cdname+" flying from "+name, 5);
					this.$TowbarLostCargo[id] += cargo.t;
				}
			} else if( rnd < 0.02 ) { //15% fuel explosion
				if( ( Math.floor( ship.entityPersonality / 328 ) + 1 ) / 10 //from ShipVersion oxp
					> this.$TowbarCurrentRep || Math.random() < 0.1 ) { //10% always failed
					player.consoleMessage("Warning: fuel leaking from the towed ship!", 10);
					this.$Towbar_ShipExplode();
				} else player.consoleMessage("You prevented a fuel explosion in "+name+"!", 10);
			} else if( rnd < 0.0205 ) { //0.75% trumbles
				if( ( Math.floor( ship.entityPersonality / 328 ) + 1 ) / 10 //from ShipVersion oxp
					> this.$TowbarCurrentRep || Math.random() < 0.1 ) { //10% always failed
					player.ship.awardEquipment("EQ_TRUMBLE");
					player.ship.awardEquipment("EQ_TRUMBLE");
//					player.ship.useSpecialCargo("EQ_TRUMBLE");//removed, can not clear at sun
					player.consoleMessage("You found Trumbles in "+name+"!", 10);
				} else player.consoleMessage("You prevented a Trumble invasion from "+name+"!", 10);
			} /*else if( rnd < 0.02 ) { //15% - removed due to cargo transfer considered safe
				var cargo = this.$Towbar_Cargo( ship );
				cargo.t = Math.max( 1, Math.floor( cargo.t / assist ) ); //Miner Assist catch the half
				//if(system && system.mainStation && system.mainStation.market)
				//	cargo.name = system.mainStation.market[cargo.name].displayName;
				player.consoleMessage(cargo.t+"t "+cargo.name+" destroyed in "+name, 5);
				this.$TowbarLostCargo[id] += cargo.t;
			}*/
		} else if( c < this.$TowbarMining1 / assist + 0.9 ) { ///do not use equal to avoid rounding problems
			var prime = "";
			if( this.$TowbarMiner == 1 )//cargo only mode finished?
				prime = ", mining need to prime Towbar and press mode";
			var cargo = this.$Towbar_Cargo( ship );
			var t = cargo.t;
			var cdname = cargo.name;
			//if(system && system.mainStation && system.mainStation.market)
			//	cdname = system.mainStation.market[cargo.name].displayName;
			if( cargo.t > 0 ) {
				if( player.ship.specialCargo ) t = 0;
				else if( player.ship.cargoSpaceAvailable < cargo.t ) {
					t = player.ship.cargoSpaceAvailable; //t=0 if no space or trumbles
					this.$TowbarLostCargo[id] += t;
				} else {
					this.$TowbarLostCargo[id] = 100000;
				}
				if( t > 0 ) player.ship.manifest[cargo.name] += t;
			}
			if( player.ship.specialCargo ) cargo.v = 0; //trumble bugfix
			if( cargo.v > 0 ) {
				var vdname = cargo.vname;
				//if(system && system.mainStation && system.mainStation.market)
				//	vdname = system.mainStation.market[cargo.vname].displayName;
				player.ship.manifest[cargo.vname] += cargo.v;
				this.$TowbarLostCargo[id] = 100000;
			}
			if( t > 0 || cargo.v > 0 || ship.fuel > 0 && player.ship.fuel < 7 ) {
				var msg = "";
				if( t > 0 ) msg += t+"t "+cdname;
				if( t > 0 && cargo.v > 0 ) {
					if( ship.fuel > 0  && player.ship.fuel < 7 ) msg += ", ";
					else msg += " and ";
				}
				if( cargo.v > 0 ) msg += cargo.v+cargo.vunit+" "+vdname;
				if( ship.fuel > 0  && player.ship.fuel < 7 ) {
					if( t > 0 || cargo.v > 0 ) msg += " and ";
					var f = Math.round( Math.min( 7 - player.ship.fuel, ship.fuel ) * 10 ) / 10 ;
					player.ship.fuel += f;
					msg += f+"ly fuel";
					ship.fuel = 0;
				}
				player.consoleMessage(msg+" transferred from "+name+prime, 10 / assist );
			} else {
				this.$TowbarLostCargo[id] = 100000;
				if(cargo.t == 0) player.consoleMessage("Ship Miner cannot found any cargo in "+name+prime, 10);
			}
			if( cargo.t > 0 ) {
				if( t < cargo.t || player.ship.specialCargo ) { //not all cargo was it into the player storage
					var rem = cargo.t - t;
					if( this.$TowbarMiner == 3 ) { //salvager mode drop out remains
						this.$TowbarLostCargo[id] += rem;
						for( var j = 0; j < rem; j += 20 ) {
							var lost = Math.min(20, rem - j); //max. 20t in one large cargo box
							var a = null;
							if( lost > 1 ) a = ship.spawnOne("more-cargo");
							if( a && a.isValid ) {
								if( !a.script ) a.script = "oolite-default-ship-script.js";
								if( a.script ) {
									a.script.$cargo = lost;
									a.script.$cargoname = cargo.name;
								}
								a.displayName = lost+"t "+cdname;
							} else {
								var a = this.$TowbarShip.spawn("cargopod", Math.min(64, lost));
								if( a ) for(var i=0; i<a.length; i++) {
									if( a[i] && a[i].isValid ) {
										a[i].setCargo(cargo.name);
										a[i].displayName = "1t "+cdname;
										a[i].bounty = 0;//to avoid * mark
									}
								}
							}
						}
						player.consoleMessage( "Your store is full, "
							+rem+"t "+cdname+" flying from "+name, 10 / assist  );
					} else player.consoleMessage( "Your store is full, some cargo leaved in "+name,
						10 / assist );
				}
			}
			if( this.$TowbarLostCargo[id] >= 100000
				|| this.$TowbarLostCargo[id] >= Math.min(120, ship.cargoSpaceCapacity) ) {
				if( worldScripts["detectors"] && ship.script )
					ship.script.$Detectors_Origname = "Empty " + ship.script.$Detectors_Origname;
				else ship.displayName = "Empty " + ship.displayName;
				if( !ship.script ) ship.script = "oolite-default-ship-script.js";
				if( ship.script ) ship.script.$TowbarEmptyShip = 1;
			}
			if( rnd < 0.2 ) this.$Towbar_ParcelAddUrgent();  //urgent parcel found (20%)
			else if( rnd < 0.4 ) this.$Towbar_ParcelAdd(); //parcel found (20%)
			else if( rnd < 0.45 ) {//both found (5%)
				this.$Towbar_ParcelAdd();
				this.$Towbar_ParcelAddUrgent();
			} else if( rnd < 0.48 ) {//two normal parcel found (3%)
				this.$Towbar_ParcelAdd();
				this.$Towbar_ParcelAdd();
			} else if( rnd < 0.5 ) {//three normal parcel found (2%)
				this.$Towbar_ParcelAdd();
				this.$Towbar_ParcelAdd();
				this.$Towbar_ParcelAdd();
			}
		} else if( c < this.$TowbarMining2 / assist - 0.1 ) { ///do not use equal to avoid rounding problems
			var t = Math.round( (1-Math.random()) * this.$Towbar_Computers( ship ) );
			t = Math.max( 1, Math.floor( t / assist ) ); //Miner Assist catch the half
			if( t > 0 ) {
				if( rnd < 0.01 ) { //10%
					player.consoleMessage(t+"t Computers flying from "+name, 5);
					var p = ship.spawn("cargopod", Math.min(64, t) );
					if( p ) for(var i=0; i<p.length; i++) {
						if( p[i] && p[i].isValid ) {
							p[i].setCargo("computers");
							p[i].displayName = "1t Computers";
							p[i].bounty = 0;//to avoid * mark with detectors oxp
						}
					}
					this.$TowbarLostComps[id] += t;
				} else if( rnd < 0.02 ) { //10%
					player.consoleMessage(t+"t Computers destroyed in "+name, 5);
					this.$TowbarLostComps[id] += t;
				}
			}
		} else if( c < this.$TowbarMining2 / assist + 0.9 ) { ///do not use equal to avoid rounding problems
			var t = this.$Towbar_Computers( ship );
			if( t > 0 && !player.ship.specialCargo ) {
				if( player.ship.cargoSpaceAvailable < t || player.ship.specialCargo ) {
					var rem = t - player.ship.cargoSpaceAvailable;
					if( this.$TowbarMiner == 3 ) { //salvager mode drop out remains
						var p = ship.spawn("cargopod", Math.min(64, rem) );
						if( p ) for(var i=0; i<p.length; i++) {
							if( p[i] && p[i].isValid ) {
								p[i].setCargo("computers");
								p[i].displayName = "1t Computers";
								p[i].bounty = 0;//to avoid * mark with detectors oxp
							}
						}
						player.consoleMessage( "Your store is full, "
							+rem+"t Computers flying from "+name, 10 );
					} else player.consoleMessage( "Your store is full, "
							+rem+"t Computers leaved in "+name, 10 );
					if( !player.ship.specialCargo ) {
						t = player.ship.cargoSpaceAvailable;
						this.$TowbarLostComps[id] += t;
					}
				} else {
					this.$TowbarLostComps[id] = 100000;
					player.consoleMessage(t+"t Computers mined from "+name, 10);
					player.ship.manifest.computers += t;
				}
			} else player.consoleMessage("Ship Miner cannot get usable computer from "+name, 10 );
		} else if( c < this.$TowbarMining3 / assist - 0.1 ) {
			var eq = ship.equipment;
			if( rnd < 0.005 * eq.length ) { //10%/equipment
				if( eq.length > 1 ) { //leave in the last equipment
					var num = Math.round(Math.random() * (eq.length-1));
					player.consoleMessage(eq[num].name+" flying from "+name, 5);
					var a = ship.spawnOne("one-machinery");
					if( a && a.isValid ) {
						if( !a.script ) a.script = "oolite-default-ship-script.js";
						if( a.script ) {
							a.script.$cargo = 1;
							a.script.$cargoname = "machinery";
						}
						a.displayName = "Damaged "+eq[num].name;
					} else ship.spawnOne("more-wreckage");
					ship.removeEquipment(eq[num].equipmentKey);
				} else { //fly an alloy instead
					player.consoleMessage("Alloy flying from "+name, 5);
					this.$TowbarShip.spawnOne("alloy");//fragmented from the ship during equipment mining
					this.$TowbarLostAlloys[id]++;
				}
			} else if( rnd < 0.01 * eq.length ) { //10%/equipment
				if( eq.length > 1 && assist < 2 ) { //leave in the last equipment and miner assist help
					var num = Math.round(Math.random() * (eq.length-1));
					ship.removeEquipment(eq[num].equipmentKey);
					player.consoleMessage(eq[num].name+" destroyed in "+name, 5);
				} else { //destroy an alloy instead
					player.consoleMessage("Alloy destroyed in "+name, 5);
					this.$TowbarLostAlloys[id]++;
				}
			} else if( rnd < 0.015 * eq.length ) { //10%/equipment
				player.consoleMessage("Alloy flying from "+name, 5);
				this.$TowbarShip.spawnOne("alloy");//fragmented from the ship during equipment mining
				this.$TowbarLostAlloys[id]++;
			} else if( rnd < 0.02 * eq.length ) { //10%/equipment
				player.consoleMessage("Alloy destroyed in "+name, 5);
				this.$TowbarLostAlloys[id]++;
			}
		} else if( c < this.$TowbarMining3 / assist + 0.9 ) {
			///get as many Machinery as equal with the average payment for the equipments and weapons in the dock
			var eprice = this.$Towbar_EqPrice( ship );
			var eq = ship.equipment; //local variable, faster
			var t = Math.floor( eprice / 56.4 ); //56.4=average Machinery price
			if( t > 0 && !player.ship.specialCargo ) {
				if( player.ship.cargoSpaceAvailable < t || player.ship.specialCargo ) {
					var rem = t - player.ship.cargoSpaceAvailable;
					if( player.ship.specialCargo ) rem = t;
					if( this.$TowbarMiner == 3 ) { //salvager mode drop out remains
						for( var j = 0; j < rem; j += 20 ) {
							var lost = Math.min(20, rem - j); //max. 20t in one
							var a = null;
							if( lost > 1 ) a = ship.spawnOne("more-machinery");
							if( a && a.isValid ) {
								if( !a.script ) a.script = "oolite-default-ship-script.js";
								if( a.script ) {
									a.script.$cargo = lost;
									a.script.$cargoname = "machinery";
								}
								a.displayName = lost+"t Machinery";
							} else {
								var a = this.$TowbarShip.spawn("cargopod", Math.min(64, lost));
								if( a ) for(var i=0; i<a.length; i++) {
									if( a[i] && a[i].isValid ) {
										a[i].setCargo("machinery");
										a[i].displayName = "1t Machinery";
										a[i].bounty = 0;//to avoid * mark
									}
								}
							}
						}
						player.consoleMessage( "Your store is full, "
							+rem+"t Machinery flying from "+name, 10 );
						t = player.ship.cargoSpaceAvailable;
						var i = -1; //remove as many equipments as enough to cover the mined machinery
						while( eq[++i] && eprice - this.$Towbar_EqPrice( ship ) > t * 56.4 )
							ship.removeEquipment(eq[i].equipmentKey);
					} else player.consoleMessage( "Your store is full, "
						+rem+"t Machinery leaved in "+name, 10 );
				} else {
					var i = -1;
					while( eq[++i] ) ship.removeEquipment(eq[i].equipmentKey);
//					ship.forwardWeapon = null; //not enough, need to check mined status also at payment
//					ship.aftWeapon = null;
					ship.forwardWeapon = ship.aftWeapon = "EQ_WEAPON_NONE";//still not enough
					player.consoleMessage(t+"t Machinery pulled out from "+name, 10);
				}
				if( t > 0 && !player.ship.specialCargo ) player.ship.manifest.machinery += t;
			} else player.consoleMessage("Ship Miner cannot found equipments in "+name, 10 );
		} else {
			var prevlost = 0;
			if(this.$TowbarLostAlloys[id] > 0) prevlost = this.$TowbarLostAlloys[id];
			//last part is not time limited, will finish when 2/3 of the alloys mined
			var left = this.$Towbar_AlloysFromShip( ship ) / 1.5 - prevlost;
			if( left >= 2 ) { //dismantle the ship hull to alloys
				if( Math.round(c) == Math.floor( c / 4 * assist ) * 4 / assist ) {
					//in every 4. seconds (or 2. with assist)
					var t = Math.round(Math.min(left, 2 + Math.round(Math.random() * 11)));
					if( Math.random() > 0.1 ) {
						if( t > 10 ) { //can not scoop too large pieces
							ship.spawnOne("more-wreckage");
							player.consoleMessage(t+"t Wreckage flying from "+name, 5);
						} else {
							if( t > 7 || player.ship.cargoSpaceAvailable < t 
								|| player.ship.specialCargo ) {
								player.consoleMessage(t+"t Alloys flying from "+name, 5);
								var a = ship.spawnOne("more-alloys");
								if( a && a.isValid ) {
									a.displayName = t+"t Alloys";
									if( !a.script )
										a.script = "oolite-default-ship-script.js";
									if( a.script ) {
										a.script.$cargo = t;
										a.script.$cargoname = "alloys";
									}
								} else ship.spawn("alloy",  Math.min(64, t));
							} else { //max. 7t can be transferred without scooping
								if( t > 0 ) player.ship.manifest.alloys += t;
								player.consoleMessage(t+"t Alloys get off from "+name, 5);
								if( player.ship.cargoSpaceAvailable < 1 )
									player.consoleMessage( "Your store is full", 10);
							}
						}
					} else player.consoleMessage(t+"t Alloys destroyed in "+name, 5);
					this.$TowbarLostAlloys[id] += t;
				}
			} else { //finished mining, increase reputation
				this.$TowbarMiningC[id] = -1;
				if( missionVariables.$TowbarShips > 0 ) missionVariables.$TowbarShips++; //salvager reputation
				else missionVariables.$TowbarShips = 1;
				player.commsMessage("Ship Miner finished your "+missionVariables.$TowbarShips+". ship.", 10);
				if( worldScripts["detectors"] && ship.script ) {
					var o = ship.script.$Detectors_Origname;
					if( o.substring(0, 6) == "Empty " ) o = o.substring(6); //without "Empty " prefix
					ship.script.$Detectors_Origname = "Mined " + o;
				} else {
					var o = ship.displayName;
					if( o.substring(0, 6) == "Empty " ) o = o.substring(6); //without "Empty " prefix
					ship.displayName = "Mined " + o;
				}
				if( !ship.script ) ship.script = "oolite-default-ship-script.js";
				if( ship.script ) ship.script.$TowbarMinedShip = 1; //avoid rep. again in dock
				this.$Towbar_UpdateReputation(false);
				if( this.$Towbar_IsHeavyShip( ship ) ) this.$Towbar_Release(); //release heavy ship
			}
		}

		if( rnd > 0.9999 ) { //damage in every 10000 sec but in average much earlier
 			if( assist > 1 ) {
				player.ship.setEquipmentStatus("EQ_SHIPMINERASSISTANT", "EQUIPMENT_DAMAGED")
				player.consoleMessage("Ship Miner Asssistant damaged", 10);
				this.$TowbarSound2.play();
			}
		} else if( rnd > 0.9998 ) { //in every 10000 sec but in average much earlier
			player.ship.setEquipmentStatus("EQ_SHIPMINER", "EQUIPMENT_DAMAGED")
			player.consoleMessage("Ship Miner damaged, mining halted", 10);
			this.$TowbarSound2.play();
		} else if( rnd > 0.9997 ) { //in every 10000 sec but in average much earlier
			this.$Towbar_ShipExplode();
			player.ship.removeEquipment("EQ_SHIPMINER");
			player.consoleMessage("Ship Miner destroyed by the explosion!", 10);
			this.$TowbarSound2.playOrRepeat();
	 		if( assist > 1 && rnd > 0.99975 ) { //50% to damage assistant which works out of the ship
				player.ship.setEquipmentStatus("EQ_SHIPMINERASSISTANT", "EQUIPMENT_DAMAGED")
				player.consoleMessage("Ship Miner Asssistant damaged", 10);
				this.$TowbarSound2.playOrRepeat();
			}
		}
	}

	this.$TowbarPrevSpeed = player.ship.speed;//save to can check the difference next time
	this.$TowbarPrevMiner = this.$TowbarMiner; //save ship miner mode to detect change

	if(!system) return;
	var st = system.filteredEntities(this, this.$Towbar_IsDerelict, player.ship, 100); //find derelict ship in 100m
	if( st.length > 0 ) this.$Towbar_SteerTo( st[0] ); //Towbar lock on

	ship = this.$TowbarShip; //maybe new if just locked
	if( ship && ship.isValid ) this.$Towbar_ScannerColor(ship);
	
}

this.$Towbar_UpdateReputation = function( startup ) {
	var ws = worldScripts["towbar"];
	var rep = 0;
	var ships = missionVariables.$TowbarShips;
//	ships = 13;//debug
	if( ships > 1 ) { //at least 2 ships to avoid "1 ships" message
		if( ships < 4 ) rep = 1;
		else if( ships < 8 ) rep = 2;
		else if( ships < 16 ) rep = 3;
		else if( ships < 32 ) rep = 4;
		else if( ships < 64 ) rep = 5;
		else if( ships < 128 ) rep = 6;
		else if( ships < 1000 ) rep = 7;
		else if( ships < 3000 ) rep = 8;
		else if( ships < 6000 ) rep = 9;
		else rep = 10; //Elite Salvager
	}
	var msg = expandMissionText("display_reputation_salvage_" + rep, { ships: ships });
	mission.setInstructions( msg, "towbar" );//show in F5-F5 screen, need specify ws name due to Tug Drone payout call
	if( !startup && ws.$TowbarCurrentRep < rep ) {
		ws.$TowbarSound5.play();//applause
		player.commsMessage("You reached the "+rep+". level in Salvager Rankings!", 10);
		player.consoleMessage(msg, 10);
	}
	ws.$TowbarCurrentRep = rep;
}

this.$Towbar_VClear = function() {
		if( this.$TowbarV ) this.$TowbarV.remove();
		this.$TowbarV = null;
		this.$TowbarPrevOri = null;
}